/* ############################################################
 * Programm zur Dekompressionsberechnung nach Bhlman ZH-L16
 * Autor: Helmut Fahrion
 * E-Mail: hefa@gmx.net
 * Lizenz: GPL
 * Datum: 11.2003
 * ############################################################
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <math.h>
#include <getopt.h>

// Zentrale Versionsnummer
#include "deko-version.h"

#define FN2 79			// N2-Anteil im Atemgas
#define FHE 79			// HE-Anteil im Atemgas
#define FO2 21			// O2-Anteil im Atemgas
#define FALLES 100

#define WDAMPF 0.0623		// Druck des Wasserdampfes, wirkt dem Wasserdruck entgegen. Ist ev. nur 0.054 ??
#define NULLMAX 99		// Nullzeit nicht lnger als 99
#define MAXDS   200		// Maximale Anzahl der Dekostopps, sollte reichen!

#define true 1
#define false 0

// Erdbeschleunigung und Umgebungsdruck als Konstanen Einfhren
#define ERD_BESCH 10.0

char *dekoerror =
  "Fehler: kein Dekostop fr die erforderliche Tiefe vorhanden!\n";

// Gewebekonstanten fr Stickstoff

// Halbwertzeiten von 4min bis 635min
double t05N2[] = { 4.0, 8.0, 12.5, 18.5, 27.0, 38.3, 54.3, 77.0,
  109.0, 146.0, 187.0, 239.0, 305.0, 390.0, 498.0, 635.0
};

// Koeffizient a
double aN2[] = {
  1.2599, 1.0, 0.8618, 0.7562, 0.662, 0.5043, 0.441, 0.4, 0.375, 0.35,
  0.3295, 0.3065, 0.2835, 0.261, 0.248, 0.2327
};

// Koeffizient b
double bN2[] = {
  0.505, 0.6514, 0.7222, 0.7825, 0.8126, 0.8434, 0.8693, 0.891,
  0.9092, 0.9222, 0.9319, 0.9403, 0.9477, 0.9544, 0.9602, 0.9653
};

// Gewebekonstanten fr Helium
// Halbwertzeiten von 1.51 min bis 240 min
double t05He[] = {
  1.51, 3.02, 4.72, 6.99, 10.21, 14.48, 20.53, 29.11, 41.2, 55.2, 70.7, 90.2,
  115.2, 147.2, 188.0, 240.0
};

// Koeffizient a
double aHe[] = {
  1.7424, 1.383, 1.1919, 1.0458, 0.922, 0.8205, 0.7305, 0.6502,
  0.595, 0.5545, 0.5333, 0.5189, 0.5181, 0.5176, 0.5172, 0.5119
};

// Koeffizient b
double bHe[] = {
  0.4245, 0.5747, 0.6527, 0.7223, 0.7582, 0.7957, 0.8279, 0.8553,
  0.8757, 0.8903, 0.8997, 0.9073, 0.9122, 0.9171, 0.9217, 0.9267
};


// Ab welcher Tiefe Helium?
double TriDepth = 40.0,
// Ab welcher Tiefe o2?, wird auch mal konfigurierbar gemacht
  O2Depth = 6.0,
// Eingabeparameter fr ppO2 
  konst_ppo2 = 0.0, tolerant = 0.0,
// Faktor fr die Deepstopberechnung, gibt an auf welches Volumen 
// das Gas ansteigen kann bis der erste Tiefe Stop erfolgt
  deep_faktor = 2.0,
// Widerholungstauchgang
  wdhgtg = 0.0;


int dive_depth = 40, dive_time = 20, air_pressure = 1013,	// durchschnittlich in Deutschland
  amv = 20, o2 = 20, figN2 = 79, figHe = 79, isHe = 0, dekoo2 = 0, isO2 =
  0, nurhelium = 0, profout = 0, saverest = 0, realprofil =
  0, deepstop = 0, deep_time = 2, deko_step = 3, koeff_b = 0,
  print_maxsaett = 0, deep_treppe = 0, deko6m = 0;

// Datei fr Tauchprofil Ausgabe
FILE *OUT;
#define outfile  "deko-profil.txt"

// Ausgabe fr Reststtigung
FILE *SAET;
#define restfile  "deko-rest.txt"

// File fr die Nachrechnung realer Tauchgnge
FILE *REAL;
#define realfile  "deko-real.txt"

// k in der Sttigungsberechnung
enum conste_k
{
  konservativ = 1,
  aladin,
  buelmann,
}
ke;

// kein Parameter, grere Dekowerte
int konstante = konservativ;

// Berechnet die Restnullzeit
int
auftauchm_restnullzeit (double dp, double tE,
			const double pig_N2[16], const double pig_He[16],
			int *r0s);
void init_const (double pig_N2[16], double pig_He[16], int deko[MAXDS]);


// Berechnet den O2 Prozentsatz bei einem konstantan ppO2
double
calc_o2_konstppo2 (double d)
{
  return ((konst_ppo2 / (d / 10.0 + 1.0)) * 100.0);
}

// Anteile der Gasbestandteile berechnen
void
init_gas (double d)
{
  double rest;

  if (nurhelium)
    {
      figN2 = 0;
      if (konst_ppo2)
	{
	  o2 = calc_o2_konstppo2 (d);
	  figHe = 100.0 - o2;
	}
      else
	{
	  figHe = isHe;
	  o2 = 100.0 - isHe;
	}
    }
  else
    {
      if (konst_ppo2)
	{
	  o2 = calc_o2_konstppo2 (d);
	  figHe = isHe;
	  figN2 = 100.0 - figHe - o2;
	}
      else
	{
	  figHe = isHe;
	  rest = FALLES - figHe;
	  figN2 = ((double) FN2 / 100.0) * rest;
	  o2 = ((double) FO2 / 100.0) * rest;
	}
    }
}


// Luft 
void
luft (double d)
{
  if (konst_ppo2)
    {
      // Tiefe zurckrechnen
      o2 = calc_o2_konstppo2 (d);
      figN2 = 100.0 - o2;
      figHe = 0;
    }
  else
    {
      figHe = 0;
      figN2 = FN2;
      o2 = FO2;
    }
}

// Nitrox
void
nitrox (void)
{
  o2 = isO2;
  figN2 = 100 - o2;
  figHe = 0;
}

void
calc_depth (double d)
{
  // Sauerstoff
  if ((d <= O2Depth) && dekoo2)
    {
      figHe = figN2 = 0;
      o2 = 100;
    }
  else
    // +++ Tiefe noch mitberechnen +++
  if (isHe && (d > TriDepth))
    {
      init_gas (d);
    }
  else if (isO2)
    {
      // wechsel zu Nitrox
      nitrox ();
    }
  else
    luft (d);
}

// Wasserdruck in gegebener Tiefe berechnen 
double
wasser_druck (double f_depth)
{
  return ((f_depth / ERD_BESCH) + (((double) air_pressure) / 1000.0));
}

// Gewebepartialdrcke fr Inertgas ausrechnen
void
rechne_inertgasdruck (char profile, int print_restnull, double depth,
		      double tE, double pig_N2[16], double pig_He[16])
{
  int x, r0 = NULLMAX;
  double piigN2, piigHe, exponent, k, pamb;


  // Inertgaspartialdruck fr alle 16 Kompartimente berechnen
  switch (konstante)
    {
    case buelmann:
      // nach Bhlmann nicht vorhanden
      k = -1.0;
      break;
    case aladin:
      k = -1.667;
      break;
    default:
    case konservativ:
      // Konstante abhngig von der Tauchtiefe definieren, hnliche Werte wie in Deko2000
      if (depth <= 3)
	k = -1.4;
      else if (depth <= 6)
	k = -2.1;
      else if (depth <= 9)
	k = -2.5;
      else if (depth <= 12)
	k = -2.6;
      else if (depth <= 15)
	k = -2.7;
      else
	k = -3.0;
      break;
    }

  // Inspiratorischer Inertgasdruck in den Lungenalveolen
  // Bhlmann Gleichung (I)
  pamb = wasser_druck (depth) - WDAMPF;
  piigN2 = pamb * ((double) figN2) / 100.0;
  piigHe = pamb * ((double) figHe) / 100.0;

  for (x = 0; x <= 15; x++)
    {
      // Inertgasdruck im Kompartiment (mit spec. t1/2) nach geg.
      // Expositionszeit Bhlmann Gleichung (II)
      exponent = tE / t05N2[x] * k;
      pig_N2[x] += (piigN2 - pig_N2[x]) * (1 - exp (exponent));

      exponent = tE / t05He[x] * k;
      pig_He[x] += (piigHe - pig_He[x]) * (1 - exp (exponent));
    }
  // Ausgabe Profile
  if (profile)
    {
      if (print_restnull == -1)
	{
	  x = auftauchm_restnullzeit (depth, tE, pig_N2, pig_He, &r0);
	  fprintf (OUT, "%3.2f %3.2f %3d %3d %3d %2d %2d\n",
		   depth, tE, x, r0, o2, figN2, figHe);
	}
      else
	fprintf (OUT, "%3.2f %3.2f %3d 0 %3d %2d %2d\n",
		 depth, tE, print_restnull, o2, figN2, figHe);
    }
}

long
getdecostep (double mtr, int deko[MAXDS])
{
  int x;

  // Grenzbereich bei Dekopflicht bei 2-1m, gehe runter 
  // auf 3m und mache 1min und mache Deko!
  if (deko[0] >= mtr)
    {
      return 0;
    }

  // Suche Dekostopp raus, immer Tiefe + 1!
  for (x = 0; x < MAXDS; x++)
    {
      //if (deko[x - 1] < Metr && deko[x] >= Metr)
      if (deko[x] < mtr && deko[x + 1] >= mtr)
	return (x + 1);
    }

  // Fehler keine Deko gefunden, bestimmt zu tief,
  // so das die Gre des Array nicht reicht.
  return -1;
}


/* Berechnen des Umgebungsdruckes, bei dem gerade noch keine Gasblasen auftreten
 * (abhngig von der bisher erfolgten Aufsttigung der Gewebe mit N2). */
double
blasendruck (const double pig_N2[16], const double pig_He[16])
{
  int x;
  double pamb, fn2, fhe, a, b, c;
  double max_p = 0.0;

  fn2 = figN2 / 100 + 0.001;
  fhe = figHe / 100 + 0.001;

  // Aus der Schleife rausgenommen
  c = fn2 + fhe;

  // Rechne wie bie N2
  for (x = 0; x <= 15; x++)
    {
      // Gewichtung von He und N2
      a = (aN2[x] * fn2 + aHe[x] * fhe) / c;
      b = (bN2[x] * fn2 + bHe[x] * fhe) / c;

      // Bhlmann Formel (III) Fr die Dekompressionsphase 
      pamb = ((pig_N2[x] + pig_He[x]) - a) * b;

      if (pamb > max_p)
	max_p = pamb;
    }
  return max_p;
}

void
save_rest (const double pig_N2[16], const double pig_He[16])
{
  int x;
  char *linie = "#-------------------------\n";

  SAET = fopen (restfile, "w+");

  if (errno)
    {
      perror ("Fehler beim Anlegen der Ausgabedatei.");
      fprintf (stderr, "Datei: %s\n", restfile);
      exit (-1);
    }

  fprintf (SAET, "# Konstanten fuer pig_teN2\n");
  fprintf (SAET, linie);

  for (x = 0; x <= 15; x++)
    fprintf (SAET, "%f ", pig_N2[x]);

  fprintf (SAET, "\n");


  fprintf (SAET, "# Konstanten fuer pig_teHe\n");
  fprintf (SAET, linie);

  for (x = 0; x <= 15; x++)
    fprintf (SAET, "%f ", pig_He[x]);

  fprintf (SAET, "\n");

  fclose (SAET);
}


/*
 * Berechne Reststtigung
 * 
 * Keine Sttigung ist mehr vorhanden wenn
 * der Stickstoff <= figN2 also 0.79 ist
 * bei Helium eigentlich fast Null, hier wird der 
 * Wasserdampfdruck genommen.
 * 
 */
int
calc_restsaettigung (double *restmax, double N2[16], double He[16])
{
  int x, t1;
  double d, r, rmax, n2ue;

  // Gas ist Luft
  luft (0.0);
  // keine Ausgabe
  profout = 0;

  // Schleife
  t1 = 0;
  rmax = r = 0.0;
  n2ue = figN2 / 100.0;
  d = n2ue;
  while (d > 0)
    {
      rechne_inertgasdruck (0, 0, 0, 60, N2, He);	// eine Stunde
      t1++;

      d = 0.0;
      for (x = 0; x <= 15; x++)
	{
	  // Stickstoffberdruck zu 79%
	  if (N2[x] > n2ue)
	    {
	      r = N2[x] - n2ue;
	      if (r > rmax)
		rmax = r;

	      d = N2[x];
	    }
	  // Heliumberdruck zu 0 %
	  if (He[x] > WDAMPF)
	    {
	      r = He[x];
	      if (r > rmax)
		rmax = r;

	      d = He[x];
	    }
	}
    }

  *restmax = rmax;
  return t1;
}


/*
 * Berechnet die Flugverbotszeit
 * berechnet die Stickstoffabgabe bei normalem Umgebungsdruck
 * und testet ob noch eine Reststtigung bei der Flughhe vorhanden ist.
 */

int
calc_fly (int begin, double N2[16], double He[16])
{
  double rs;
  int rest, ap, nfly;

  // Luftdruck merken  
  ap = air_pressure;

  rest = begin;
  rechne_inertgasdruck (0, 0, 0, 60 * (rest - 1), N2, He);

  nfly = rest - 1;

  while (rest > 1)
    {
      air_pressure = ap;	// rechne mit einer Hhe von etwa 2200 Meter
      rechne_inertgasdruck (0, 0, 0, 60, N2, He);	// eine Stunde

      // ist auf 2200 Meter Hhe etwa 1/4 niedriger als auf Meereshhe
      air_pressure = ap - ap / 4;

      rest = calc_restsaettigung (&rs, N2, He);
      nfly++;
    }

  // Luftdruck zurckschreiben  
  air_pressure = ap;

  // keine negative Flugverbotszeit
  if (nfly < 0)
    nfly = 0;

  return nfly;
}

/* 
 * Aufsttigung je nach Daten im Profile - File 
 */

double
CalcProfilfile (double pig_N2[16], double pig_He[16])
{
  int BUFS = 256;
  char buffer[BUFS];
  double tiefe, sz, ez;

  REAL = fopen (realfile, "r+");

  if (errno)
    {
      perror ("Fehler beim ffnen der Eingabedatei.");
      fprintf (stderr, "Datei: %s\n", realfile);
      exit (-1);
    }

  sz = 0.0;
  while (fgets (buffer, BUFS, REAL))
    {
      if (buffer[0] != '#')
	{
	  sscanf (buffer, "%lf %lf\n", &tiefe, &ez);

	  // Berechne gleich die Sttigung
	  calc_depth (tiefe);	// Tiefenabhngig auf anderes Gas wechseln
	  rechne_inertgasdruck (profout, -1, tiefe, ez - sz, pig_N2, pig_He);

	  sz = ez;
	}
    }

  fclose (REAL);
  return tiefe;
}

/*
 *
 * Leider ist diese Berechnung nicht so gut geeignet wie die 'Iterative Form'.
 * Bei groen Sttigungswerten, sowie bei Tiefen von kleiner 10m, problematisch.
 * Die Ergebnisse werden negativ, die Nullzeit kann nicht berechnet werden.
 *
 */

int
restnullzeit_log (double dp, const double pig_N2[16], const double pig_He[16])
{
  int t0min = NULLMAX;		// nicht Lnger als 3 h

  int x;
  double xN2, xHe, piigN2, piigHe, pamb, te, airp;

  pamb = wasser_druck (dp) - WDAMPF;
  piigN2 = pamb * ((double) figN2) / 100.0;
  piigHe = pamb * ((double) figHe) / 100.0;

  airp = (air_pressure / 1000.0);
  for (x = 0; x <= 15; x++)
    {
      // Keine Division / 0
      // Beim Tauchen in geringen Tiefen kann es passieren, da der 
      // Umgebungsdruck des Inertgases auch noch bei Sttigung der Gewebe 
      // niedriger ist als der an der Oberflche noch tolerierte maximale 
      // Druck. D.h. hlt sich ein Gewebe unendlich lange in einer solchen 
      // Tiefe auf und ist damit vollstndig gesttigt, kann immer noch 
      // direkt zur Oberflche aufgestiegen werden. Damit ist die Nullzeit 
      // dieses Gewebes aber unendlich lang. Dies gilt zumindest fr das 
      // Modell ZH-L16.
      if ((te = piigN2 - pig_N2[x]) && figN2)
	{
	  xN2 = 1.0 - (((airp / bN2[x] + aN2[x]) - pig_N2[x]) / te);

	  // Grenzwertig !!!
	  // Hlt sich ein schon dekompressionspflichtiger Taucher in einer 
	  // Tiefe auf, in der er sich noch immer weiter aufsttigt, 
	  // d.h. der Teildruck der Inertgase im Gewebe ist kleiner als der 
	  // Teildruck der Inertgase im Atemgas, so kann der Taucher niemals 
	  // zurck in eine positive Nullzeit kommen. Mit anderen Worten: 
	  // Der Taucher bleibt weiter dekompressionspflichtig und die gesamte 
	  // Dekompressionzeit wird grer.
	  if (xN2 >= 1.0)
	    {
	      // t0min = 0;
	      // Brauche garnicht weiterrechen da die Nullzeit sowieso schon
	      // 0 ist.
	      return 0;
	    }
	  else
	    {
	      if (xN2 > 0.0)	// N2 Logarithmieren mglich?
		{
		  //te = log (xN2) / log (2) * -t05N2[x];
		  te = log (xN2) / 0.30102999566398 * -t05N2[x];
		  if (te < t0min)
		    t0min = te;
		}
	    }
	}

      if ((te = piigHe - pig_He[x]) && figHe)
	{
	  xHe = 1.0 - ((airp / bHe[x] + aHe[x] - pig_He[x]) / te);
	  if (xHe >= 1.0)
	    {
	      //t0min = 0;
	      return 0;
	    }
	  else
	    {
	      if (xHe > 0)	// He Logarithmieren mglich? 
		{
		  te = log (xHe) / 0.30102999566398 * -t05He[x];
		  if (te < t0min)
		    t0min = te;
		}
	    }
	}
    }
  return t0min;
}

// in welchen Schritten die Nullzeit wieder erhht wird
#define NULLSTEP 3

// Berechnung der Restnullzeit
int
auftauchmeter_schleife (double dp, double N2[16], double He[16])
{
  do
    {
      rechne_inertgasdruck (0, 0, dp, 0.1, N2, He);
      if (blasendruck (N2, He) >= (wasser_druck (dp) - WDAMPF))
	{
	  return (floor (dp - 1.0) + 0.5);	// Auftauchen bis dp
	}
      else
	{
	  dp -= 1.0;
	}
    }
  while (dp > 0.0);

  return 0;			// Auftauchen bis 0
}

int
restnull_check (double dp, double tE, double N2[16], double He[16])
{
  int ret;
  // bleibe da !
  rechne_inertgasdruck (0, 0, dp, tE, N2, He);
  ret = auftauchmeter_schleife (dp, N2, He);
  return ret;
}

int
auftauchm_restnullzeit (double dp, double tE, const double pig_N2[16],
			const double pig_He[16], int *restnullzeit_save)
{
  int x, save_prof, t0m;
  double N2[16], He[16];

  // Sttigungen sichern
  memcpy (N2, pig_N2, sizeof (double) * 16);
  memcpy (He, pig_He, sizeof (double) * 16);

  // keine Ausgabe
  save_prof = profout;
  profout = 0;

  t0m = auftauchmeter_schleife (dp, N2, He);

  // Ist man im Bereich der "richtigen" Nullzeit berechne mit log 
  x = restnullzeit_log (dp, pig_N2, pig_He);
  if (x > 0)
    {
      *restnullzeit_save = x;
      goto ende;
    }

/*
 * Kann ich bis 0m auftauchen, dann berechne ich die Zeit wielange ich 
 * auf tiefe bleiben kann, bis das Auftauchen bis 0m nicht mehr mglich ist.
 *
 */

  // Das hier ist also die "schmutzige" Nullzeitberechnung
  // Da restnullzeit_log() eine Nullzeit 0 ausgegeben hat, kann man hier 
  // iterativ probieren ob man ohne Dekostopp zur Oberflche kommt so das
  // keines der Gewebe die Zulssige Gasspannung berschreitet.
  //
  // Das gilt auch fr Tiefen in denen der Taucher sich immer mehr 
  // aufsttigt und eigentlich nur mit einem Dekostopp zur Oberflche
  // zurckkehren kann. 
  // Bei der Iterativen Form wird die Entsttigung des Tauchers whrend des 
  // Aufstieges mitgerechnet. 

  if (!t0m)
    {
      // Sttigungen zurckschreiben
      memcpy (N2, pig_N2, sizeof (double) * 16);
      memcpy (He, pig_He, sizeof (double) * 16);

      // Wenn die Nullzeit nicht groesser ist dann ist Sie halt kleiner
      while (*restnullzeit_save)
	{
	  if (*restnullzeit_save < 1)
	    {
	      *restnullzeit_save = 0;
	      goto ende;
	    }

	  x = restnull_check (dp, *restnullzeit_save, N2, He);

	  if (!x)		// ging nicht bis nach oben!
	    {
	      goto ende;
	    }
	  *restnullzeit_save -= NULLSTEP;
	}
    }
  // keine Nullzeit
  *restnullzeit_save = 0;

ende:
  // Profile wieder ausgeben.
  profout = save_prof;
  return t0m;
}


// --- MOD Berechnung ---
double
o2mod (double o2)
{
  // MOD
  return (((1.6 / (o2 / 100.0)) - 1.0) * 10.0);
}

double
o2mind (double o2)
{
  // 16% O2 oder 2000m Hhe
  return (((0.16 / (o2 / 100.0)) - 1.0) * 10.0);
}


// --- Ausgabeformatierung ---
void
printfline (char *str)
{
  if (strlen (str) > 1)
    {
      printf ("---- %s ----\n", str);
    }
  else
    {
      printf ("----------------------------------------\n");
    }
}

/*
 * Dekoberechnung fr Rechtecktauchgang
 */

void
tauchgang (void)
{
  double p_next, p_next2, save_ppo2, p_max, t_decotime, t_abtime, t_asctime,
    o2air, air, gair, d, ges_time, auf_speed, ab_speed;

// Die Array fr die Gewebesttigungen !!!
  double pig_teN2[16], pig_teHe[16],
    // zum zwischenspeichern
    N2[16], He[16];

  char deko_stop, verl_deco;
  int deep_depth, t1, x, cur_deco_depth,
    dekostopp, cur_deco_step, no_deco, t_elaps, deko[MAXDS];
  //double ds1, ds2;
  // deep stop arrays
#define DSANZ 10
  int ds[DSANZ];
  int dsnext;


  // array null setzen
  memset (ds, 0, sizeof (int) * DSANZ);

  // Init
  verl_deco = 0;
  cur_deco_step = no_deco = 1;

  dsnext = deep_depth = t_elaps = 0;
  p_max = t_decotime = t_abtime = t_asctime = air = gair = o2air = 0.0;

  // Konstanten berechnen, und Reststtigungen einlesen
  init_const (pig_teN2, pig_teHe, deko);

  // Tiefe und Zeit wird auch bei realprofile mit angegeben!
  printfline ("");
  printf ("Tauchtiefe:	%4u m\n", dive_depth);
  printf ("Tauchzeit (GZ):%4u min\n", dive_time);


  if (nurhelium)
    printf ("Helium:	%4d %% He\n", isHe);
  else if (isHe)
    printf ("Trimix:	%4d %% He\n", isHe);

  if (konst_ppo2)
    printf ("ppO2:\t\t %1.2f \n", konst_ppo2);

  if (dekoo2)
    printf ("Sauerstoff: \t%3d %% O2 zur Deko.\n", 100);

  if (isO2)
    printf ("Nitrox:	%4d %% O2\n", isO2);

  printf ("AMV:		%4d barl\n", amv);
  printf ("Luftdruck:	%4d mbar\n", air_pressure);
  if (wdhgtg > 0.0)
    {
      printf ("Oberfl.Pause:\t %2.1f h\n", wdhgtg);
    }

  printfline ("");
  dekostopp = dive_depth;

  // Berechnung des Tiefen Stops, bei einer Volumenvergrsserung
  // der ev. Gasblschen auf 1,5 Fachen des Volumens
  // druck auf tiefe =  tiefe / 10 + 1
  //  druck ausdehnung  =  druck auf tiefe / faktor
  // tiefe = (druck ausdehnung - 1 ) * 10

  // 1.0 hat kein Sinn, denn es ist die Tauchtiefe
  /*
     if (deep_faktor > 1.0)
     {
     ds2 = ds1 = (((dive_depth / 10.0 + 1.0) / deep_faktor) - 1.0) * 10.0;
     deep_depth = (int) floor (ds1 + 0.5);

     // < 3 ist ja Unsinn
     if (deep_depth >= 3)
     {
     ds2 = (((ds1 / 10.0 + 1.0) / deep_faktor) - 1.0) * 10.0;
     deep_depth2 = (int) floor (ds2 + 0.5);
     if (deep_depth2 < 3)
     {
     deep_depth2 = deep_depth;
     }
     }
     else
     {
     deep_depth2 = deep_depth = deepstop = 0;
     }
     }
     else
     {
     deep_depth2 = deep_depth = deepstop = 0;
     }
   */
  if (deep_faktor > 1.0)
    {
      deep_depth = ds[0] =
	(int)
	floor ((((wasser_druck (dive_depth) / deep_faktor) -
		 (air_pressure / 1000.0)) * ERD_BESCH) + 0.5);

      // Stop 2 bis oben 
      for (x = 0; x < (DSANZ - 1); x++)
	{
	  if (ds[x] > 3)	// der vorhergehende muss kleiner 3 sein
	    {
	      ds[x + 1] =
		(int)
		floor ((((wasser_druck ((double) ds[x]) / deep_faktor) -
			 (air_pressure / 1000.0)) * ERD_BESCH) + 0.5);
	      if (ds[x + 1] < 0)
		ds[x + 1] = 0;
	    }
	}
    }

  dsnext = 0;			// nchste Teife stopp ist ...

// debug
/*
      fprintf(stderr, "Berechnete Deepstop Tiefen:\n");
      for (x=0; x < DSANZ; x++)
      {
 	fprintf(stderr, "d=%dm\n", ds[x]);
      }
*/

  // Bei Widerholungstauchgang
  // Die Entsttigung an Oberflche berechnen 
  rechne_inertgasdruck (0, 0, 0.0, wdhgtg * 60.0, pig_teN2, pig_teHe);

  // Abstiegsgeschwindigkeit 20m/min
  ab_speed = 1.0 / 2.0 / 10.0;	// min / meter

  if (realprofil)
    {
      // Aufstiegsphase beginnt beim letzten Wert.
      dive_depth = CalcProfilfile (pig_teN2, pig_teHe);
    }
  else
    {
// -----------  ABTAUCHPHASE  -------------------------

// Berechnung nach Bhlmann (nicht integrierend)
      for (t1 = 1; t1 <= dekostopp; t1++)
	{
	  calc_depth ((double) t1);	// Tiefenabhngig auf anderes Gas wechseln
	  rechne_inertgasdruck (profout, -1, (double) t1, ab_speed, pig_teN2,
				pig_teHe);
	  t_abtime += ab_speed;

	  // bis auf Tridepth mit air
	  if (t1 <= TriDepth)
	    air += amv * wasser_druck ((double) t1) * ab_speed;
	  else
	    gair += amv * wasser_druck ((double) t1) * ab_speed;
	}

      // Abstiegsphase abziehen
      dekostopp = dive_time - (dekostopp * 0.1);

// ------------- GRUNDZEIT --------------------------
      calc_depth ((double) dive_depth);	// Tiefenabhngig auf anderes Gas wechseln
      for (t1 = 1; t1 <= dekostopp; t1++)
	{
	  rechne_inertgasdruck (profout, -1, (double) dive_depth, 1.0,
				pig_teN2, pig_teHe);

	}
      // zeit auf tiefe
      gair += amv * wasser_druck ((double) dive_depth) * dekostopp;
    }

// --------- Aufstiegsphase (y=10m/min.) -------------
  t1 = dive_depth;

  // -------- Problem ist ein Realprofil bei dem der Aufstieg dabei ist, 
  // --- somit wird kein Aufstieg gerechnet ! --------------------------
  if (!realprofil || t1 > 3.0)
    do
      {
	calc_depth ((double) t1);	// Tiefenabhngig auf anderes Gas wechsel

	// Variable Aufstiegsgeschwindigkeit
	// 0  - 10 m  7m/min
	// 10 - 20 m 10m/min
	// 20 - 40 m 20m/min
	if (t1 >= 20)
	  {
	    auf_speed = 1.0 / 20.0;
	  }
	else
	  {
	    if (t1 >= 10)
	      {
		auf_speed = 1.0 / 10.0;
	      }
	    else
	      auf_speed = 1.0 / 7.0;
	  }


	// Der Druck mu hier schon getestet werden, sonst gibt es Zacke
	// im Profil
	if (blasendruck (pig_teN2, pig_teHe) <
	    (wasser_druck ((double) t1) - WDAMPF))
	  {
	    rechne_inertgasdruck (profout, -1, (double) t1, auf_speed,
				  pig_teN2, pig_teHe);
	    deko_stop = 0;
	  }
	else
	  {
	    deko_stop = 1;
	  }

	// 1m / aufspeed
	t_asctime += auf_speed;

	// luftverbrauch wrend aufstieg
	gair += amv * wasser_druck ((double) t1) * auf_speed;

	if (deepstop)
	  {
	    // Test ob Deko tiefer als Deepstop ist !
	    if (deko_stop)
	      {
		cur_deco_step = getdecostep ((double) t1, deko);
		if (cur_deco_step < 0)
		  {
		    fprintf (stderr, dekoerror);
		    printf (dekoerror);
		    exit (-1);
		  }
		cur_deco_depth = deko[cur_deco_step];
		if (cur_deco_depth >= deep_depth)
		  {
		    deep_depth = 0;
		    // 1. Dekostopp wird lnger da deepsto in dekotiefe
		    // fllt, also verlngere Dekostop auf dieser Tiefe
		    verl_deco = deep_time;
		  }
	      }

	    // der letzte Deepstop  
	    if (t1 == deep_depth)
	      {
		printfline ("tiefer Stop");
		printf ("%3d min auf %3u m\n", deep_time, deep_depth);

		// bleibe maximal innerhalb der Nullzeit auf der Tiefe
		rechne_inertgasdruck (profout, -1, deep_depth, deep_time,
				      pig_teN2, pig_teHe);

		t_decotime += deep_time;	// mu hier dazugehhlt werden

		// luftverbrauch wrend deepstop
		gair += amv * wasser_druck ((double) t1) * deep_time;

		// Deepstop Treppen 
		if (deep_treppe)
		  {
		    deep_depth -= 3;
		    if (deep_depth < 3)
		      {
			// Kein Deepstop mehr   
			deepstop = 0;
		      }
		  }
		else
		  {
		    if (dsnext < (DSANZ - 1))
		      {
			deep_depth = ds[++dsnext];
		      }
		    else
		      {
			// Kein Deepstop mehr   
			deepstop = 0;
		      }
		  }

	      }
	  }

	// War der umgebungsdruck >= dem wasser_druck, dann Deko           
	if (deko_stop)
	  {
	    if (profout)
	      {
		fprintf (OUT, "# --- Deko erforderlich - m: %d ---\n", t1);
	      }

	    printfline ("Deko ist erforderlich!");
	    cur_deco_step = getdecostep ((double) t1, deko);
	    if (cur_deco_step < 0)
	      {
		fprintf (stderr, dekoerror);
		printf (dekoerror);
		exit (-1);
	      }
	    cur_deco_depth = deko[cur_deco_step];
	    printf ("1. Deko-Stopp:	%3u m\n", cur_deco_depth);
	    printf ("Deko-Stopps:	%3u\n", cur_deco_step + 1);
	    printf ("Aufstiegszeit:	%3.0f min\n", (t_asctime + 0.5));
	    no_deco = 0;
	    t1 = 0;
	  }
	else
	  {
	    // steige auf
	    t1--;
	  }
      }
    while (t1 > 0);

//----------------- Ende Aufstiegsphase -------------------
  if (no_deco == 1)
    {
      printfline ("Keine Deko ist erforderlich!");

      // Grundzeit, rechne zur Sicherheit Gesamtzeit * Druck * AMV
      air =
	amv * wasser_druck ((double) dive_depth) * (dive_time + t_asctime);

      printf ("Gesamte Luftmenge:	%5.1f ltr.\n", air);

      goto ausgabe;
    }
  printfline ("Dekompressionsphase");

  // Erwarteter Druck auf nchster Deko-Stufe 
  p_next = wasser_druck ((double) deko[cur_deco_step]);

  /* Gesamt-Deko berechnen */
  printf ("Deko Stopps:\n");

  double step_time;

  while (1)
    {
      t_elaps++;		// iterativ im Minutentakt berechnen

      d = (double) deko[cur_deco_step];
      calc_depth (d);		// Tiefenabhngig auf anderes Gas wechseln

      step_time = 1.0;

      rechne_inertgasdruck (profout, (int) d, d, step_time, pig_teN2,
			    pig_teHe);

      if (!verl_deco)
	{
	  // gewebe umgebungsdruck
	  p_max = blasendruck (pig_teN2, pig_teHe);
	}
      else
	{
	  // Verlngere Dekostopp um Tiefenstopp
	  verl_deco--;
	  continue;
	}

      if (p_max > p_next)	// Verbleiben
	{
	  continue;
	}
      else
	{
	  // verbleibe auf taucherwunsch auf 6m anstelle aufzusteigen
	  if (deko6m && (cur_deco_step == 1))
	    {
	      p_next2 = wasser_druck ((double) deko[0]) - WDAMPF;
	      if (p_max > p_next2)
		{
		  continue;
		}
	    }
	}

      // Wurde beim Verlngerten Stopp die Dekopflicht beendet, dann berspringe
      // die nchsten Routinen

      if (cur_deco_step >= 0)
	{

	  if (cur_deco_step > 0)
	    dekostopp = (long) floor ((double) deko[cur_deco_step - 1] + 0.5);
	  else
	    dekostopp = deko[cur_deco_step];

	  // Zwischenentsttigung whrend Aufsteigens zur nchsten Deko-Stufe
	  // berechnen v=10m/min.
	  for (t1 = (long) floor ((double) deko[cur_deco_step] + 0.5);
	       t1 >= dekostopp; t1--)
	    {
	      rechne_inertgasdruck (profout, (int) d, (double) t1, 0.1,
				    pig_teN2, pig_teHe);
	      t_abtime += 0.1;	// Halt hier noch addieren
	    }

	  // bei deko mit 100% o2
	  if (o2 == 100)
	    {
	      o2air +=
		amv * wasser_druck ((double) deko[cur_deco_step]) * t_elaps;
	    }
	  else
	    {
	      air +=
		amv * wasser_druck ((double) deko[cur_deco_step]) * t_elaps;
	    }
	  printf ("%3d min auf %3u m ", t_elaps, deko[cur_deco_step]);

	  printf ("O2=%2d%% ", o2);
	  if (figHe)
	    printf ("He=%2d%% ", figHe);
	  printf ("N2=%2d%% ", figN2);
	  printf ("\n");

	  t_decotime += t_elaps;
	  t_elaps = 0;
	  cur_deco_step--;
	}
      else
	{
	  // Deko wurde mit tiefen Stop beendet
	}

      if ((cur_deco_step < 0) || (cur_deco_step >= MAXDS))
	{
	  printfline ("Ende der Dekompressionsphase");
	  if (t_decotime > 0)
	    printf ("Deko Zeit:\t\t%5.1f min %2.1f h\n", t_decotime,
		    t_decotime / 60);
	  if (air > 0)
	    printf ("Luftmenge Deko:\t%5.1f ltr.\n", air);

	  if (o2air > 0)
	    printf ("Sauerstoffmenge Deko:\t%5.1f ltr.\n", o2air);

	  // gair = amv * wasser_druck ((double) dive_depth) * dive_time;
	  ges_time = t_abtime + t_asctime + t_decotime + dive_time;
	  printf ("Gesamte Zeit:\t\t%5.1f min %2.1f h\n", ges_time,
		  (ges_time) / 60);

	  goto ausgabe;
	}
      p_next = wasser_druck ((double) deko[cur_deco_step]) - WDAMPF;
    }

ausgabe:
  printf ("Verbrauch GZ:\t\t%5.1f ltr.\n", gair);
  printf ("Gesamte Luftmenge:\t%5.1f ltr.\n", gair + air + o2air);

  // 25% Reserve dazurechnen
  x = (int) floor ((((gair + air + o2air) * 1.25) / 200.0) + 0.5);
  printf ("Volumen bei 200Bar:\t%d ltr.\n", x);

  x = (int) floor ((((gair) * 1.25) / 200.0) + 0.5);
  if (x)
    printf ("Volumen GZ bei 200Bar:\t%d ltr.\n", x);

  x = (int) floor ((((air) * 1.25) / 200.0) + 0.5);
  if (x)
    printf ("Volumen Luft bei 200Bar:\t%d ltr.\n", x);

  x = (int) floor ((((o2air) * 1.25) / 200.0) + 0.5);
  if (x)
    printf ("Volumen Sauerstoff bei 200Bar:\t%d ltr.\n", x);

  // Zur Warnung O2 Anteil bei isHe ausgeben
  // + Bar frs Mixen ausrechnen
  if (isHe)
    {
      if (nurhelium)
	{

	  printfline ("Daten zum Heliumgemisch");
	  printf ("O2/He:\t\t%2d%%/%2d%%\n", 100 - isHe, isHe);
	  printf ("O2 fuellen (200Bar):\t%2d Bar (Rest He)\n",
		  (100 - isHe) * 2);
	  printf ("He fuellen (200Bar):\t%2d Bar (Rest O2)\n", isHe * 2);

	  // 16% O2 oder 2000m Hhe
	  d = o2mind (100.0 - isHe);

	  if (d > 1.0)
	    printf ("Min.Tiefe bei 16%%O2:\t%2.0f m\n", d + 0.5);

	  // MOD
	  d = o2mod (100.0 - isHe);
	  printf ("MOD (1.6):\t\t%2.0f m\n", d);
	}
      else
	{
	  printfline ("Daten zum Helium+Luftgemisch");
	  // He > 0%, Sauerstoffanteil beim Mixen mit Luft
	  printf ("O2/He/N2:\t\t%2.1f%%/%2d%%/%2.1f%%\n",
		  (0.21 * (100.0 - isHe)), isHe, (0.79 * (100.0 - isHe)));

	  printf ("He fuellen (200Bar):\t%2.0f Bar (Rest Luft)\n",
		  isHe * 2.0);

	  // quivalente Lufttiefe zu 40m Wassertiefe
	  // d = 0.78 * (100.0 - isHe) / 100.0;  // % N2 im Atemgas
	  d = ((3.9 / (0.78 * (100.0 - isHe) / 100.0)) - 1.0) * 10.0;
	  // Tiefe im Verhltinis zu 40m (N2 pDruck 3.9Bar)
	  printf ("AENT (40m):\t\t%2.0f m\n", d);

	  // quivalente Lufttiefe auf Zieltiefe
	  // Druck auf Tiefe
	  d = (((dive_depth / 10.0 + 1.0) * (0.78 * (100.0 - isHe) / 100.0))
	       * 10.0) + 1.0;

	  // Tiefe im Verhltinis zu 40m (N2 pDruck 3.9Bar)
	  printf ("AENT (auf %dm):\t%2.0f m\n", dive_depth, d);

	  // 16% O2 oder 2000m Hhe
	  d = o2mind (0.21 * (100.0 - isHe));
	  if (d > 1.0)
	    printf ("Min.Tiefe bei 16%%O2:\t%2.0f m\n", d + 0.5);

	  // MOD
	  d = o2mod (0.21 * (100.0 - isHe));
	  printf ("MOD (1.6):\t\t%2.0f m\n", d);
	}


      if (!isO2)
	printfline ("");
    }

  if (isO2)
    {
      printfline ("Daten zum Sauerstoff Gemisch");
      // MOD
      d = o2mod (isO2);
      printf ("MOD (1.6):\t\t%2.0f m\n", d);

      // Stickstoff mit weniger O2 ? 
      d = o2mind (isO2);
      if (d > 1.0)
	printf ("Min.Tiefe bei 16%%O2:\t%2.0f m\n", d + 0.5);

      printfline ("");
    }

  // Bergsee ?
  if (air_pressure < 900)
    {
      // Barometrische Formel
      // z = [ (R * T(m) ) / g ] * ln [ p(0) / p ] 
      //
      //  wobei z die Hhe, R die Gaskonstante fr Luft (287 J/(Kg*K)), T(m) die Mitteltemperatur (in Grad Kelvin) zwischen den Druckniveaus p(0) und p, p(0) der Luftdruck am Boden, p der Luftdruck in der Hhe z und g die Schwerebeschleunigung der Erde sind.

      d = ((287.0 * (273.0 + 15.0)) / 9.81) * log (1000.0 / air_pressure);

      printf ("Hoehe:\t\t\t %4.0f m\n", d);
    }

// Falls Reststtigung gespeichert wird, berechen wir wann alles wieder raus ist
  if (saverest)
    {
      double restmax;

      // Kalkulation nicht im Profil
      if (profout)
	fclose (OUT);

      profout = 0;
      save_rest (pig_teN2, pig_teHe);	// Reststtigung

      save_ppo2 = konst_ppo2;
      konst_ppo2 = 0.0;		// Wir atmen ja Luft !

      memcpy (N2, pig_teN2, sizeof (double) * 16);
      memcpy (He, pig_teHe, sizeof (double) * 16);
      t1 = calc_restsaettigung (&restmax, N2, He);
      printf ("Reststtigung:\t\t%d h\n", t1);
      // printf ("Sttigungs Maximum:\t%1.2f\n", restmax);

      memcpy (N2, pig_teN2, sizeof (double) * 16);
      memcpy (He, pig_teHe, sizeof (double) * 16);
      t1 = calc_fly (t1 / 2, N2, He);	// beginne bei rest/2
      printf ("Flugverbot:\t\t%d h\n", t1);

      if (print_maxsaett && restmax > 0.1)
	printf ("Sttigungs Maximum:\t%1.2f\n", restmax);

      konst_ppo2 = save_ppo2;	// Zurcksetzen
    }

  printfline ("");
}

// ----------- bersttigungstolleranzen verndern ------------- 
// --- zur sicheren Seite ---
void
set_tolerant (double t)
{
  int x;
  double f;

  if (t < 1.0)
    {
      t = 1.0;
    }

  if (t > 4.0)
    {
      t = 4.0;
    }

  f = 1.0 + t / 10.0;

  for (x = 0; x <= 15; x++)
    {
      bN2[x] *= f;
      aN2[x] /= f;
    }
}

void
init_const (double pig_N2[16], double pig_He[16], int deko[MAXDS])
{
  int x;
  int BUFS = 256;
  char buffer[BUFS];
  const char *formatstr =
    "%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf\n";

  // koeffizient b wird skaliert
  double faktor[16], fak;

  if (wdhgtg > 0.0)
    {
      SAET = fopen (restfile, "r");

      if (errno)
	{
	  perror ("Fehler beim Oeffnen der Restsaettigungen.");
	  fprintf (stderr, "Datei: %s\n", restfile);
	  fclose (SAET);
	  exit (-1);
	}

      // Format ist festgelegt
      // Kopfzeile 1
      fgets (buffer, BUFS, SAET);

      // Kopfzeile 2
      fgets (buffer, BUFS, SAET);

      // Zahlen
      fgets (buffer, BUFS, SAET);

      sscanf (buffer, formatstr, &pig_N2[0],
	      &pig_N2[1], &pig_N2[2], &pig_N2[3], &pig_N2[4],
	      &pig_N2[5], &pig_N2[6], &pig_N2[7], &pig_N2[8],
	      &pig_N2[9], &pig_N2[10], &pig_N2[11], &pig_N2[12],
	      &pig_N2[13], &pig_N2[14], &pig_N2[15]);

      // Kopfzeile 1
      fgets (buffer, BUFS, SAET);

      // Kopfzeile 2
      fgets (buffer, BUFS, SAET);

      // Zahlen
      fgets (buffer, BUFS, SAET);

      sscanf (buffer, formatstr,
	      &pig_He[0], &pig_He[1], &pig_He[2], &pig_He[3],
	      &pig_He[4], &pig_He[5], &pig_He[6], &pig_He[7],
	      &pig_He[8], &pig_He[9], &pig_He[10], &pig_He[11],
	      &pig_He[12], &pig_He[13], &pig_He[14], &pig_He[15]);

      fclose (SAET);
    }
  else
    {
      for (x = 0; x <= 15; x++)
	pig_N2[x] = 0.78;
      for (x = 0; x <= 15; x++)
	pig_He[x] = 0.0;
    }

  // Toleranz, setzt die bersttigungstoleranzen
  if (tolerant > 0.0)
    set_tolerant (tolerant);

  // 16 Koeffizienten b strecken
  if (koeff_b > 0)
    {
      faktor[0] = (double) koeff_b;
      for (x = 1; x <= 15; x++)
	{
	  faktor[x] = faktor[x - 1] / 2.0;
	}
      for (x = 0; x <= 15; x++)
	{
	  fak = 1.0 + faktor[x];
	  bN2[x] *= fak;
	  aN2[x] /= fak;
	}
    }

  // Dekostufen, auch noch parametrierbar machen!
  // ev.  von 1 - 4 meter !
  for (x = 0; x < MAXDS; x++)
    deko[x] = (x + 1) * deko_step;

  // Gasmixture berechnen (o2 N2 He) entsprechend
  // der Parameter
  init_gas (0.0);

  // Ausgabedatei fr Tauchprofile
  if (profout)
    {
      OUT = fopen (outfile, "w+");

      if (errno)
	{
	  perror ("Fehler beim Anlegen der Ausgabedatei.");
	  fprintf (stderr, "Datei: %s\n", outfile);
	  exit (-1);
	}
      fprintf (OUT,
	       "# tiefe[m] zeit[min] aufstiegstiefe[m] Nullzeit[min] O2[%%] N2[%%] He[%%]\n");
      fprintf (OUT,
	       "#------------------------------------------------------------------------\n");
    }
}


// Der Hilfetext
const char usage[] =
  "Hilfe:\n"
  "---------------------------------------------------------------------\n"
  "Notation:  %s [--version|-v] [--help|-h|--hilfe]\n"
  "  xx steht fuer eine beliebige Zahl\n\n"
  "Parameter:\n"
  "--trimix xx\n"
  "-3 xx\n"
  "Rechnet mit xx %% Helium im Atemgas.\n\n"
  "--tridepth xx\n"
  "-w xx\n" "Gaswechsel ab dieser Tiefe.\n\n"
  "--helium\n"
  "Rechnet ohne Stickstoff.\n\n"
  "--dekoo2\n" "-o\n"
  "Ab einer Tiefe von 6m wird mit 100%% O2 dekomprimiert.\n\n"
  "--deko6m\n" "-6\n"
  "Der Taucher sitzt den 3m Dekostopp auf 6m ab.\n\n"
  "--nitrox xx\n"
  "-n xx\n"
  "Rechnet mit xx %% Sauerstoff im Atemgas.\n\n"
  "--amv xx\n"
  "-m xx\n"
  "Berechnet den Gasverbrauch mit der Grundlage von xx Barliter/Minute.\n\n"
  "--airp x\n"
  "-a\n"
  "Der Umgebungsluftdruck in mBar.\n\n"
  "--depth x\n"
  "-d\n"
  "Die Tauchtiefe bei Rechteckprofil.\n\n"
  "--time x\n"
  "-t\n"
  "Die Grundzeit.\n\n"
  "--profile-out\n"
  "-u\n"
  "Schreibt das Tauchprofil in die Datei %s.\n\n"
  "--save-rest\n"
  "-s\n"
  "Schreibt die Restsaettigungen in die Datei %s.\n\n"
  "--real-profil\n"
  "-r\n"
  "Liest das getauchte Profil aus der Datei %s und berechnet anschlieend das Austauchen.\n\n"
  "--ofp\n" "-p\n"
  "Oberflaechenpause zum letzten Tauchgang in h.\n\n"
  "--tolerant\n" "-l\n"
  "Toleranzfaktor, Halbwertszeiten werden Prozentual geaendert, 1-5, 3 ist ohne Aenderung 5 super sicher, 1 ist unsicher..\n\n"
  "--const\n"
  "-c\n"
  "Definiert die Konstante k bei der Berechnung der Saettigungswerte,\n"
  " 1: k wird tiefenabhngig geaendert. 1.4 bis 3.0. Ergibt Werte aehnlich der Deko-2000.\n"
  " 2: k ist 1.667. Ergibt recht praktikable Werte, aehnlich dem Aladin.\n"
  " 3: Buehlmann, zu kurze Dekowerte!\n\n"
  "--deepstop\n" "-q\n"
  "Berechnet beim Austauchen noch einen Zusaetzlichen Stop auf einem drittel der Tauchtiefe, die Dekompressionsphase bleibt meist unveraendert.\n\n"
  "--koeffb\n"
  "-b\n"
  "Gibt den Koeffizienten b aus der Buelmannschen Gleichung eine Gewichtung so das die schnellen Gewebe lansamer eintsttigen, das fhrt zu verlngerten tiefen Dekostopps. Die flacheren Stopps sind nicht betroffen."
  "--ppo2\n" "-x\n\n"
  "Berechnet den ganzen Tauchgang mit einem konstanten ppO2, wie beim Rebreather erforderlich. Dadurch aendert sich der Anteil an Inertgas und somit wird bei einem hohen ppO2 weniger Inertgas aufgenommen.\n\n"
  "--deco-step 3\n"
  "-k\n"
  "Gibt die Schrittweite der Dekostopps vor. Normalerweise aller 3m, eine feinere Abstufung von 2m oder 1m ist auch moeglich.\n\n"
  "--saett-max\n"
  "-j\n"
  "Saettigungs-Maximum ausgeben, die Sttigung des Fhrungsgewebes wird ausgegeben.\n\n"
  "--hilfe\n"
  "--help\n"
  "-h\n"
  "-?\n" "Dieser Hilfetext.\n\n"
  "--version\n" "-v\n" "Die Versionsnummer.\n\n";

static const struct option long_options[] = {
  {"deco-step", 1, 0, 'k'},
  {"tolerant", 1, 0, 'l'},
  {"helium", 0, 0, 'e'},
  {"trimix", 1, 0, '3'},
  {"tridepth", 1, 0, 'w'},
  {"dekoo2", 0, 0, 'o'},
  {"deko6m", 0, 0, '6'},
  {"nitrox", 1, 0, 'n'},
  {"amv", 1, 0, 'm'},
  {"airp", 1, 0, 'a'},
  {"depth", 1, 0, 'd'},
  {"time", 1, 0, 't'},
  {"hilfe", 0, 0, '?'},
  {"help", 0, 0, 'h'},
  {"version", 0, 0, 'v'},
  {"profile-out", 0, 0, 'u'},
  {"save-rest", 0, 0, 's'},
  {"real-profil", 0, 0, 'r'},
  {"ofp", 1, 0, 'p'},
  {"const", 1, 0, 'c'},
  {"deepstop", 0, 0, 'q'},
  {"deep-time", 1, 0, 'f'},
  {"deep-trap", 0, 0, 'g'},
  {"deep-faktor", 1, 0, 'i'},
  {"ppo2", 1, 0, 'x'},
  {"koeffb", 1, 0, 'b'},
  {"saett-max", 0, 0, 'j'},
  {0, 0, 0, 0}
};


// *********** main ***********************
int
main (int argc, char *argv[])
{
  int opt;

  // parameter auswerten
  while ((opt =
	  getopt_long (argc, argv,
		       "+6uh?vqsegkrjw:2:3:d:t:a:m:o:c:p:b:f:i:l:x:",
		       long_options, NULL)) != EOF)
    {
      switch (opt)
	{
	case '3':
	  isHe = atoi (optarg);
	  break;
	case '6':
	  deko6m = true;
	  break;
	case 'a':
	  air_pressure = atoi (optarg);
	  break;
	case 'b':
	  koeff_b = atoi (optarg);
	  break;
	case 'c':
	  konstante = atoi (optarg);
	  break;
	case 'd':
	  dive_depth = atoi (optarg);
	  break;
	case 'e':
	  nurhelium = true;
	  break;
	case 'f':
	  deep_time = atoi (optarg);
	  break;
	case 'g':
	  deep_treppe = true;
	  break;
	case '?':
	case 'h':
	  printf (usage, argv[0], outfile, restfile, realfile);
	  return 0;
	  break;
	case 'i':
	  deep_faktor = atof (optarg);
	  break;
	case 'j':
	  print_maxsaett = 1;
	  break;
	case 'k':
	  deko_step = atoi (optarg);
	  break;
	case 'l':
	  tolerant = atof (optarg);
	  break;
	case 'm':
	  amv = atoi (optarg);
	  break;
	case 'n':
	  isO2 = atoi (optarg);
	  break;
	case 'o':
	  dekoo2 = 1;
	  break;
	case 'p':
	  wdhgtg = atof (optarg);
	  break;
	case 'q':
	  deepstop = 1;
	  break;
	case 'r':
	  realprofil = 1;
	  break;
	case 's':
	  saverest = 1;
	  break;
	case 't':
	  dive_time = atoi (optarg);
	  break;
	case 'u':
	  profout = 1;
	  break;
	case 'v':
	  printf ("Version: %s\n", version);
	  return 0;
	  break;
	case 'w':
	  TriDepth = atof (optarg);
	  break;
	case 'x':
	  konst_ppo2 = atof (optarg);
	  break;
	default:
	  printf ("?? getopt returned character code 0%o ??\n", opt);
	  break;
	}
    }

  // Der Tauchgang
  tauchgang ();

  if (profout)
    fclose (OUT);

  return 0;
}
