#!/usr/bin/perl -w

############################################################
# Programm zur Dekompressionsberechnung nach Bhlman ZH-L16
# Autor: Helmut Fahrion
# E-Mail: hefa@gmx.net
# Lizenz: GPL
# Datum: 11.2003
#############################################################

use Tk;

# ---- Pfad wo die Programm und Dokudateien sind ----------------------------
$instpath	= "$ENV{HOME}/deko";

# --- Falls es keine lokale Installation gibt -------------------------------
if (! -e $instpath)
  {
    $instpath	= "/opt/deko";
  }

# ---- Farbe fr die Oberflche, sollte zum Thema passen --------------------
#$palette    = "honeydew3";
$palette    = "azure3";


$dekoprog 	= "$instpath/deko";
$doku		= "$instpath/doku";
$taucher	= "$instpath/taucher.gif";
$fischli	= "$instpath/fischli.gif";
$configfile 	= "$ENV{HOME}/.dekogui";
$udcfparser	= "$instpath/deko-udcfread.pl";
$udcffile	= "$ENV{HOME}/deko-erg.udcf";

# ---- Pfad wo die Daten hinkommen, ohne Pfad werden die Daten
# ---- dort gespeichert wo man das Programm startet -------------------------
$dekoausg 	= "deko-erg.txt";
$profilefile 	= "deko-profil.txt";
$realprofile 	= "deko-real.txt";
$tableausg 	= "deko-tabelle.txt";
$restfile	= "deko-rest.txt";

# ------------------------ Block mit Globalen Daten -------------------------
# Hauptschlich wird $EDITOR verwendet
$editor_env = "";
$editor_env = $ENV{EDITOR};
@editors = (
	"/usr/local/bin/gvim_-geometry 80x43_-R", 
	"/opt/kde3/bin/kwrite_ _ ", 
	"/usr/X11R6/bin/xedit_ _ ", 
	 "/opt/gnome2/bin/gedit_ _ ",
	);

@browsers = (
 "/opt/kde3/bin/konqueror", 
 "/usr/bin/amaya", 
 "/usr/X11R6/bin/mozilla");


$versionsfile   = "$instpath/deko-version.h";
open(D, $versionsfile) || die "Fehler beim lesen der Datei $versionsfile\n";
while ( <D> ) {
   if ( /#define version/ )
   {
     /"(.*)"/;
     $version = $1;
   }
}
close(D);

$witho2 	= 0;
$deko6m		= 0;
$divetime 	= 15;
$divedepth 	= 45;
# durchschnitt in Deutschland
$airpressure 	= 1013;
$nitrox 	= 0;
$trimix 	= 0;
$trimixdepth	= 0;
$amv 		= 15;
$wdhg 		= 0;
$kons 		= 0;
$profile 	= 0;
$dekostep 	= 0;
$konstante 	= 0;
$withdeep 	= 1;
$deeptime	= 0;		# Zeit fr Tiefenstopp 1
$deeptrap	= 0;		# Deepstops Treppenfrmig alle 3m
$deepfaktor 	= 0.0;		# Faktor fr tolerierte Volumenvergrerung
$ppo2 		= 0.0;		# Gleitkommazahl von 0.20 bis 1.60 
$koeffb		= 0;		# Faktor fr den Koeffizienten b aus Gleichung 
$udcf_param	= 0; 		# 0=keine Ausgabe
$nurhe		= 0; 		# Rechnet nicht mit Luft+He sondern nur mit He und O2
$saettmax	= 0;		# Ausgabe des am meisten Gesttigten Gewebes


$statustext = "Statuszeile...";

# ----- Array fr die Dekostops ----------
@dekostopps = ();

# ----- Drucker Daten -----
$LPRCMD = "lpr -s";
$LPRFILENAME = "deko-print.ps";

# ----- erzeuge ein Hauptfenster -----------
$top = MainWindow->new;
$top->title("X Deko");
$top->setPalette($palette);


# ---------------------------- Arbeitsbereiche ------------------------------

@main_packer = qw/-side top -fill x/;
$menubar = $top->Frame(-relief => 'raised', -borderwidth => 2)->pack(@main_packer);

@work_packer = qw/-side top -fill both -expand yes/;
$work = $top->Frame(-relief => 'sunken', -borderwidth => 2)->pack(@work_packer);

my $text = $work->Scrolled(qw/Text -width 50 -setgrid false -scrollbars e/);
			
$text->pack(qw/-side left -fill y/);

# ---------------------------- Grafikflche ---------------------------------

my(@pu) = qw/-side top -expand yes -fill both -pady 1m/;
my $f1 = $work->Frame(-relief => 'raised', -borderwidth => 1)->pack(@pu);

my $diag = $f1->Scrolled(qw/Canvas -width 14c -height 10c -relief sunken 
-borderwidth 2 -scrollbars se  -scrollregion / => ['0', '0', '28c', '20c']);
$diag->pack(@pu); 

@pu = qw/-side top -fill x -pady 1m/;
my $frame2 = $f1->Frame(-relief => 'sunken', -borderwidth => 1)->pack(@pu);

my $neubutton =	$frame2->Button(
		-text => 'Neu',
		-command => [\&neuertg])
		->pack(qw/-side left/);
$neubutton->focus();

my $printbutton = 	$frame2->Button(
		-text => 'Drucken',
		-command => [\&printdiag, $diag])
		->pack(qw/-side left/);
					
my $printb2 = 	$frame2->Button(
		-text => 'Dateidruck',
		-command => [\&printdatei, $diag])
		->pack(qw/-side left/);
					
my $printasc = 	$frame2->Button(
		-text => 'Textdruck',
		-command => [\&printasc])
		->pack(qw/-side left/);
					
my $printtab = 	$frame2->Button(
		-text => 'Tabelle',
		-command => [\&print_tabelle])
		->pack(qw/-side left/);

my $printtabd =	$frame2->Button(
		-text => 'Tabelle speichern',
	      -command => [\&save_tabelle])
	      ->pack(qw/-side left/);


diagramm();

# ----- Statuszeile ----- 
@status_packer = qw/-side top -fill x -anchor sw/;
$status = $top->Frame(-relief => 'raised', -borderwidth => 1)->pack(@status_packer);
my $status_l1 = $status->Entry(-relief => 'sunken', -width => 60,
    -textvariable => \$statustext)->pack(@status_packer);

# ------------------- Menue ----------------------------------------------

@menue_packer = qw/-side left -padx 1m/;
my $f = $menubar->Menubutton(qw/-text Datei -underline 0 -menuitems/ =>
   [
   	[Button => 'Neuer Tauchgang...', -underline => 0,  -command => [\&neuertg]],
   	[Button => 'Tabelle ausgeben...', -underline => 8,   -command => [\&tabelle]],
        [Button => 'Tabelle speichern',   -command => [\&print_tabelle, 'Print']],
 	[Separator => ''],
 	[Button => 'Grafik drucken',   -command => [\&printdiag, 'Print']],
 	[Button => 'Grafik in Datei',   -command => [\&printdatei, 'Print']],
 	[Button => 'ASCII-Druck',   -command => [\&printasc, 'Print']],
 	[Button => 'Tabelle drucken',   -command => [\&print_tabelle, 'Print']],
 	[Separator => ''],
 	[Button => 'UDCF ansehen',   -command => [\&udcf_ansehen]],
 	[Button => 'Tabelle bearbeiten',   -command => 
	    [\&show_editor, "$tableausg", ""]],
 	[Separator => ''],
 	[Button => 'Quit/Ende', -underline => 0, -command => [$top => 'exit']],
  ])->pack(@menue_packer);

my $f2 = $menubar->Menubutton(qw/-text Einstellungen -underline 0 -menuitems/ =>
	[
	[Button => 'Konstanten ...', -underline => 0,  -command => [\&config, 'Open']],
	[Separator => ''],
	[Button => 'Speichern', -underline => 0,  -command => [\&save]],
	[Button => 'Laden', -underline => 0,   -command => [\&read]],
	[Separator => ''],
	[Button => 'Profil extern...',    -command => 
	    [\&show_editor, "$instpath/$realprofile", ""]],
	[Button => 'Profil intern...',    -command => [\&editintern]],
	[Button => 'Profil udcf...',    -command => [\&edit_udcf]],
  ])->pack(@menue_packer);


# --- Je nach Einstellungen ist das udcf File gepackt oder nicht. ---
sub udcf_ansehen
{
  if ($udcf_param == 1)
  {
    # --- nix erzeugt nix ansehen ! ---
    my $button = $top->messageBox('-icon' => 'info', -type => 'OK',
	 -title => 'MessageBox', -message => "In Einstellungen \"kein\" UDCF File eingestellt!");
    return 1;	 
  }
  if ($udcf_param == 3)
  {
    # --- gepackt -> auspacken ---
    # $udcffile	= "$ENV{HOME}/deko-erg.udcf";
    system "bunzip2 $udcffile.bz2";
    my $button = $top->messageBox('-icon' => 'info', -type => 'OK',
	 -title => 'MessageBox', -message => "UDCF File ausgepackt!");
  }
  show_editor("$udcffile", "");
}


@menue_packer = qw/-side right -padx 3m/;
my $m = $menubar->Menubutton(qw/-text Hilfe -underline 0 -menuitems/ =>
   [
      [Button => 'Version ...',    -command => [\&show_version]],
	[Separator => ''],
      [Button => 'Quelle deko.c ...', -command => [\&show_editor, "$instpath/doku/deko.c", "-R"]],
      [Button => 'Quelle gui ...', -command => [\&show_editor, "$instpath/doku/deko-tk.pl", "-R"]],
      [Button => 'UDCF-Writer ...', -command => [\&show_editor, "$instpath/doku/deko-udcf.pl", "-R"]],
      [Button => 'UDCF-Reader...', -command => [\&show_editor, "$instpath/doku/deko-udcfread.pl", "-R"]],
	[Separator => ''],
      [Button => 'Programmdoku ...',    -command => [\&show_doku]],
      [Button => 'README ...',    -command => [\&show_editor, 
      				"$instpath/README", ""]],
	[Separator => ''],
      [Button => 'Lizenz ...',    -command => [\&show_editor, "$doku/gpl.txt"]],
	 ])->pack(@menue_packer);

# ------ Externe UDCF Datei ffnen ------------------------------------
sub edit_udcf {
   @types = ( ["UDCF Files", '.udcf'], ["All files",		'*']);
   $file = $top->getOpenFile(-filetypes => \@types);

   # --- udcf parser starten ---
   system "perl $udcfparser $file";
}

# ------  Editor starten ------------------------------------------
sub show_editor {
  my $file;
  my $opt;
  my $ro;

  # Ist das Programm vorhanden ?
	foreach (@editors)
	{
	  ($file, $opt, $ro) = split(/_/, $_);

	  if ( -e $file)
	  {
            if (!($_[1] cmp "-R"))
	     {
	        system "$file $opt $ro $_[0] &"; # hier wird der komplette Aufruf genommen
	     }
	     else
	     {
	        system "$file $ro $_[0] &"; # hier wird der komplette Aufruf genommen
	     }

	     goto ende; 
	  }
  	}
ende:

}

# ------ Browser starten --------------------------------------------

sub show_doku {
  # Ist das Programm vorhanden ?
	foreach (@browsers)
	{
	  if ( -e $_)
	  {
	     system "$_"." $doku/index.html &";
	     goto ende; 
	  }
  	}
ende:
}


# ------ Dialogbox Einstellungen --------------------------------------------

sub config {
	my $toplevel = MainWindow->new;
	$toplevel->setPalette($palette);
	$toplevel->title("Einstellungen");
	
	my(@pu) = qw/-side top -expand yes -pady 1m/;
	my $toplabel = $toplevel->Label(-text => 'Ihre Einstellungen:')->pack(@pu);
	
	my $topl = $toplevel->Frame(-relief => 'raised', -borderwidth => 1)->pack(@pu);
	
	my(@sl) = qw/-side left -padx 2m -fill x -anchor nw/;
	my $f3 = $topl->Frame->pack(@pu);
	my $left_l3 = $f3->Label(-text => 'Luftdruck [hPa]:')->pack(@sl);
	my $luftdruck = $f3->Entry(-relief => 'sunken', -width => 4,
	    -textvariable => \$airpressure)->pack(@sl);
	
	my $f10 = $topl->Frame->pack(@pu);
	my $left_l10 = $f10->Label(-text => 'AMV [barl/min]:')->pack(@sl);
	my $amve = $f10->Entry(-relief => 'sunken', -width => 4,
	    -textvariable => \$amv	)->pack(@sl);
	
	my $f11 = $topl->Frame->pack(@pu);
	my $left_l11 = $f11->Label(-text => 'Konstant ppO2 [0.20..1.6]:')->pack(@sl);
	my $eppo2 = $f11->Entry(-relief => 'sunken', -width => 4,
	    -textvariable => \$ppo2 )->pack(@sl);
												
	my $f12 = $topl->Frame->pack(@pu);
	my $left_l12 = $f12->Label(-text => 'Konservativ [1..4] (1):')->pack(@sl);
	my $konservativ = $f12->Entry(-relief => 'sunken', -width => 4,
	    -textvariable => \$kons	)->pack(@sl);
	
	
	my $f1b = $topl->Frame->pack(@pu);
	my $left_l1b = $f1b->Label(-text => 'Koeffizient b strecken (2):')->pack(@sl);
	my $koeffizientb = $f1b->Entry(-relief => 'sunken', -width => 4,
	    -textvariable => \$koeffb	)->pack(@sl);

	    
	my $f13 = $topl->Frame->pack(@pu);
	my $left_l13=$f13->Label(-text => 'Dekostop Tiefe [m](3)')->pack(@sl);
	my $dekostepe = $f13->Entry(-relief => 'sunken', -width => 4,
	    -textvariable => \$dekostep	)->pack(@sl);
	

	my $fdeeptime = $topl->Frame->pack(@pu);
	my $ldeeptime=$fdeeptime->Label(-text => 'Tiefenstop Dauer [min](2):')->pack(@sl);
	my $e1deeptime = $fdeeptime->Entry(-relief => 'sunken', -width => 4,
	    -textvariable => \$deeptime	)->pack(@sl);
	
	my $fdeepfaktor = $topl->Frame->pack(@pu);
	my $ldeepfaktor=$fdeepfaktor->Label(-text => 'Tiefenstop Vol.Faktor (2.0):')->pack(@sl);
	my $e1deepfaktor = $fdeepfaktor->Entry(-relief => 'sunken', -width => 4,
	    -textvariable => \$deepfaktor	)->pack(@sl);

	my $ftdeepfaktor = $topl->Frame->pack(@pu);
	#my $ltdeepfaktor=$ftdeepfaktor->Label(-text => 'Tiefenstop Treppe:')->pack(@sl);
	my $ltdeepfaktor = $ftdeepfaktor->Checkbutton(-text => 'Tiefenstop Treppe?', -variable => \$deeptrap, -relief => 'flat')->pack(@sl);
	
	my $f1 = $topl->Frame->pack(@pu);
	my $left_l1 = $f1->Label(-text => "Dekompressions-\nFaktor k:")->pack(@sl);

	my(@sa) = qw/-side top -padx 2m -anchor w/;
	my @alg = ("1.4 bis 3.0","Normal 1.7","* 1.0 unverndert");
	foreach my $p (1, 2, 3) {
	    $f1->Radiobutton(-relief => 'sunken', 
		-text => $alg[$p-1],
		-variable => \$konstante,
		-value => $p)->pack(@sa);
	}


	my $f5 = $topl->Frame->pack(@pu);
	my $saettout = $f5->Checkbutton(-text => 'Ausgabe Fhrungsgewebe', -variable => \$saettmax, -relief => 'flat')->pack(@sa);

	

	my $top2 = $toplevel->Frame(-relief => 'raised', -borderwidth => 1)->pack(@pu);
	my $f122 = $top2->Frame->pack(@pu);
	my $left_l122 = $f122->Label(-text => 'Druck-Kommando:')->pack(@sl);
	my $lpre = $f122->Entry(-relief => 'sunken', -width => 8,
		-textvariable => \$LPRCMD)->pack(@sl);

	my $top3 = $toplevel->Frame(-relief => 'raised', -borderwidth => 1)->pack(@pu);
	my $f123 = $top3->Frame->pack(@pu);
	my $left_l123 = $f123->Label(-text => 'UDCF:')->pack(@sl);
	my @udcfp = (qw/kein ascii bzip2/);
	foreach my $p (1, 2, 3) {
	    $f123->Radiobutton(-relief => 'sunken', 
		-text => $udcfp[$p-1],
		-variable => \$udcf_param,
		-value => $p)->pack(@sl);
	}
	
	my $f124 = $top3->Frame->pack(@pu);
	my $left_l4 = $f124->Label(-text => 'Dateiname:')->pack(@sl);
	my $ofilee = $f124->Entry(-relief => 'sunken', -width => 22,
	    -textvariable => \$udcffile)->pack(@sl);
		
	my $f7 = $toplevel->Frame->pack(@pu);
	my $ok = $f7->Button(-text => 'Fertig',
		-command => sub { destroy $toplevel })
		->pack(qw/-side left -padx 2m -pady 2m/);
}


# ------ Dialogbox Tauchgangseingabe ----------------------------------------

# ------ schaltet String um --------------- 
sub swstr {
  if ($nurhe == 1)
  {
    $trimixstr = "Helium [%He]:";
  } else
  {
    $trimixstr = "Luft mit Helium [%He]:";
  }
}

sub neuertg {
	my $toplevel = MainWindow->new;
	$toplevel->setPalette($palette);

	# --- default werte ---
	swstr();
	
	$toplevel->title("Neuer Tauchgang");
	
	my(@pu) = qw/-side top -expand yes -pady 1m/;
	my $toplabel = $toplevel->Label(-text => 'Ihre Tauchgangsdaten:')->pack(@pu);
	
	my $topl = $toplevel->Frame(-relief => 'raised', -borderwidth => 1)->pack(@pu);
	
	my(@sl) = qw/-side left -padx 2m -fill x -anchor nw/;
	my $f1 = $topl->Frame->pack(@pu);
	my $left_l1 = $f1->Label(-text => 'Tauchtiefe in [m]:')->pack(@sl);
	my $tauchtiefe = $f1->Entry(-relief => 'sunken', -width => 4, -textvariable => \$divedepth,)
				->pack(@sl);
	my $f2 = $topl->Frame->pack(@pu);
	my $left_l2 = $f2->Label(-text => 'Tauchzeit in [min]:')->pack(@sl);
	my $tauchzeit = $f2->Entry(-relief => 'sunken',	-width => 4, -textvariable => \$divetime,)
					->pack(@sl);
	my $f8 = $topl->Frame->pack(@pu);
	my $trimixlabel = $f8->Label(-textvariable => \$trimixstr)->pack(@sl);
	my $trimix_e = $f8->Entry(-relief => 'sunken', -width => 4, -textvariable => \$trimix)->pack(@sl);
	
	my $f3 = $topl->Frame->pack(@pu);
	my $nurheb = $f3->Checkbutton(-text => 'Nur Helium', -variable => \$nurhe, -command => [\&swstr], -relief  => 'flat')->pack(@sl);
	
	my $f9 = $topl->Frame->pack(@pu);
	my $left_l9 = $f9->Label(-text => 'Wechsel auf [m]:')->pack(@sl);
	my $trimixdepth_e = $f9->Entry(-relief => 'sunken', -width => 4, -textvariable => \$trimixdepth)->pack(@sl);
	my $f6 = $topl->Frame->pack(@pu);
	my $left_l6 = $f6->Label(-text => 'Luft mit Sauerstoff [%O2]:')->pack(@sl);
	my $nitrox = $f6->Entry(-relief => 'sunken', -width => 4, -textvariable => \$nitrox)->pack(@sl);

	my $f4 = $topl->Frame->pack(@pu);
	my $dekoo2 = $f4->Checkbutton(-text => 'Deko mit Sauerstoff?', -variable => \$witho2, -relief  => 'flat')->pack(@sl);
	
	my $fd6 = $topl->Frame->pack(@pu);
	my $dekoo6 = $fd6->Checkbutton(-text => 'Letzten Dekostopp tiefer?', -variable => \$deko6m, -relief  => 'flat')->pack(@sl);
	
	my $fd5 = $topl->Frame->pack(@pu);
	my $deep = $fd5->Checkbutton(-text => 'Deko mit tiefen Stop', -variable => \$withdeep, -relief => 'flat')->pack(@sl);
	
	my $f11 = $topl->Frame->pack(@pu);
	my $left_l11 = $f11->Label(-text => 'Oberflchenpause [h]:')->pack(@sl);
	my $wdhg= $f11->Entry(-relief => 'sunken', -width => 4,	-textvariable => \$wdhg	)->pack(@sl);
	my $f10= $topl->Frame->pack(@pu);
	my $profile = $f10->Checkbutton(-text => 'Profildatei verwenden', -variable => \$profile, -relief => 'flat')->pack(@sl);
	my $f7 = $toplevel->Frame->pack(@pu);
	my $abbbutton = $f7->Button(-text => 'Abbruch',	-command => sub { destroy $toplevel })
	    ->pack(qw/-side left -padx 2m -pady 2m/);
	my $okbutton = 	$f7->Button( -text => 'Fertig',	-command => [\&neuer_tg])
	    ->pack(qw/-side left -padx 2m -pady 2m/);
	$okbutton->focus();    
}


# -------------- wird zur Tauchgangsberechnung, Tabellenberechnung 
#                und zur Tabellenausgabe verwendet -------------------------
sub make_parameterstring {
	$airpressure > 0 && push(@paramstr, "--airp $airpressure");
	$witho2 > 0 	 && push(@paramstr, "--dekoo2");
	$deko6m > 0 	 && push(@paramstr, "--deko6m");
	$trimix > 0 	 && push(@paramstr, "--trimix $trimix");
	$nurhe > 0       && push(@paramstr, "--helium");
	$trimixdepth > 0 && push(@paramstr, "--tridepth $trimixdepth");
	$nitrox > 0 	 && push(@paramstr, "--nitrox $nitrox");
	$amv > 0 	 && push(@paramstr, "--amv $amv");
	$wdhg > 0 	 && push(@paramstr, "--ofp $wdhg");
	$kons > 0 	 && push(@paramstr, "--tolerant $kons");
	$profile > 0 	 && push(@paramstr, "--real-profil");
	$konstante > 0 	 && push(@paramstr, "--const $konstante");
	$withdeep > 0 	 && push(@paramstr, "--deepstop");
	$koeffb > 0 	 && push(@paramstr, "--koeffb $koeffb");
	$ppo2 > 0 	 && push(@paramstr, "--ppo2 $ppo2");
	$deeptime > 0 	 && push(@paramstr, "--deep-time $deeptime");
	$deeptrap > 0 	 && push(@paramstr, "--deep-trap");
	$deepfaktor > 0  && push(@paramstr, "--deep-faktor $deepfaktor");
	$dekostep > 0 	 && push(@paramstr, "--deco-step $dekostep");
	$saettmax > 0 	 && push(@paramstr, "--saett-max");
}

# ----------- Generiere Reststtigungen fr den ersten Start ----------------
sub writerestfile_default
{
  my $x;
  open (REST, ">$restfile") || die "Kann Datei $restfile nicht ffnen!\n";
  print REST "# Konstanten fuer pig_teN2\n"; 
  print REST "#-------------------------\n";
  
  for ($x=0; $x<=16; $x++)
  {
    print REST "0.78 ";
  }
  print REST "\n";
  
  print REST "# Konstanten fuer pig_teHe\n";
  print REST "#-------------------------\n";
  
  for ($x=0; $x<=16; $x++)
  {
    print REST "0.0 ";
  }
  print REST "\n";

  close REST;
}

# ---------------------- Auswertung neuer Tauchgang -------------------------
sub neuer_tg {
	
  @paramstr = "--depth $divedepth --time $divetime ";

  make_parameterstring();

  # zum Datenaustausch wird ein File mit dem Tauchprofiel angelegt.
  push(@paramstr, "--profile-out --save-rest");

  # Sonderfall: falls eine Oberflchenpause angegeben ist und zuvor keine
  # Reststtigungen gespeichert wurden, ist die Eingabe unlogisch und das 
  # "deko" Programm beendet sich. 
  if (! -e $restfile )
  {
    writerestfile_default();
  }

  $systemstr = "$dekoprog @paramstr > $dekoausg ";
  print "systemstr: $systemstr \n";
  $statustext = "Ausfuehren: ".$systemstr;
  
  if ( -e $dekoprog )
  {
    # ----- Programmstart !!! ----------------------
    system $systemstr || die "Konnte $systemstr nicht starten!\n";
  }
  else
  {
     # ----- Compiler + Programmstart !!! ----------------------
     print "compiliere $dekoprog mit make -C $instpath!\n";
     system "make -C $instpath" || die "Konnte $dekoprog nicht compilieren!\n";
     system $systemstr || die "Konnte $systemstr nicht starten!\n";
  }

  # ----- Grafik ausgeben -----
  neue_grafik();
}


# ---------------------- Diagramm des Tauchgangs ----------------------------
sub neue_grafik {

  # ----- Ergebnisse einlesen als ASCII-Zeile ------
  my @tg_erg = ();
	
  @work_packer = qw/-side top/;

  open (IN, $dekoausg) || die "Kann Datei $dekoausg nicht ffnen!\n";
  my $x = 0;
  @dekostopps = ();
  
  while ( <IN> ) {
    push(@tg_erg, $_);
		
		# 1. Deko stopp
		if ( /min auf/ )
		{
			($zeit, $rest) = split(/min auf/, $_);
			($meter, $rest) = split(/ m/, $rest);
			
			$dekostopps[$x] = $zeit;
			$x++;
			$dekostopps[$x] = $meter;
			$x++;
		}

	}

  # damit beim zeichnen die while schleife abgebrochen wird
  $dekostopps[$x] = 0;
  close IN;

  $text->delete(qw/1.0 end/);
  $text->insert('0.0', " @tg_erg");
  $statustext = "Textinsert: $dekoausg";

  # ------- Grafikberechnung ------------------
  diagramm_tg();


  # ------- Ausgabe der UDCF Datei ------------
  # $udcf_param == 1 keine Ausgabe
  if ( $udcf_param >= 2 )
  {
    $udcfwrite	= "$instpath/deko-udcf.pl $udcffile";
    # ASCII Ausgabe
    system $udcfwrite || die "Konnte $udcfwrite nicht starten!\n";
  }  
  if ( $udcf_param == 3 )
  {
    # --- mit bzip2 packen ---
    system "bzip2 $udcffile & " || die "Konnte bzip2 nicht starten!\n";
  }


} # neuer tg


# --------------------- Speichern der globalen Daten  ------------------------
sub save {
  open(D, ">$configfile") || die "Fehler beim schreiben der Datei $configfile\n";

# --------------------------- alle globale Variablen --------------------------
  print D "\$version = \"$version\";
\$configfile = \"$configfile\";	
\$witho2 = $witho2;
\$deko6m = $deko6m;
\$divetime = $divetime;
\$divedepth = $divedepth;
\$airpressure = $airpressure;
\$nitrox = $nitrox;
\$trimix = $trimix;
\$trimixdepth = $trimixdepth;
\$amv = $amv;
\$wdhg = $wdhg;
\$profile = $profile;
\$kons = $kons;
\$LPRCMD = \"$LPRCMD\";
\$withdeep = $withdeep;
\$konstante = $konstante;
\$udcffile = \"$udcffile\";
\$ppo2 = $ppo2;
\$deeptime = $deeptime;
\$deeptrap = $deeptrap;
\$deepfaktor = $deepfaktor;
\$koeffb = $koeffb;
\$udcf_param = $udcf_param;
\$dekostep = $dekostep;
\$nurhe = $nurhe;
\$saettmax= $saettmax;
\n\n";
 
  close(D);
}

# ------------------------- Lesen der globalen Daten  -----------------------
sub read {
  open(D, "$configfile") || die "Fehler beim lesen der Datei $configfile\n";

  # ---- configfile ist gleich ein perlprogramm ! --------
  while ( <D> ) {
    eval $_;
  }
  
  close(D);
}

# ------------ Dummy Dialogbox fr die noch leeren Meneintrge -------------
sub dummy {
  my $button = $top->messageBox('-icon' => 'info', -type => 'OK',
					-title => 'MessageBox', -message => "Dummy Box");
} # end dummy

# ----------------- Dialogbox fr die Versionsnummerausgabe  ----------------
sub show_version {
  my $button = $top->messageBox(-icon => 'info', -type => 'OK', -title => 'Information', 
  				-message => "Version: $version");
} # end show_version

 
sub diagramm {
  my @p1 = qw/0.5c 0.5c/;
  my @p2 = qw/0.5c 8.0c/;
  my @p3 = qw/10.0c 8.0c/;
  my @p4 = qw/10c 0.5c/;

  # --- Skalen ---
  #                       x    y
  $diag->create("line", @p1, @p2, @p3, qw/-width 2/);
  $diag->create("line", @p3, @p4, @p1, qw/-width 1/);
  
  # Taucher oder Fisch Bild !
  $diag->Photo('bildchen', -file => "$taucher", -format => 'gif');
  $diag->Photo('fischli', -file => "$fischli");
  
  $bild_label = $diag->createImage("12c", "5c", -image => 'bildchen');
   
  1;
}


sub diagramm_tg {

  my $maxx = 10.0;
  my $maxy = 8.0;

  my $xbild = "12c";
  my $ybild = "6c";

  if ($divedepth <= 0.001)
  {
    my $error = $top->messageBox('-icon' => 'error', -type => 'OK',
		-title => 'ErrorBox', -message => "Tauchtiefe 0, Abbruch der Diagrammausgabe.");
    goto ende;		
  }
  

  if ($divedepth > 100.0)
  {
	$maxx = 20.0;
	$maxy = 16.0;
	$xbild = "22c";
	$ybild = "14c";
  }

  my @p1 = qw/0.5c 0.5c/;
  my @p2 = ("0.5c", $maxy."c");
  my @p3 = ($maxx."c", $maxy."c");
  my @p4 = ($maxx."c", "0.5c");
	
  $diag->delete('all');
	
  # --- Skalen ---
  #                       x    y
  $diag->create("line", @p1, @p2, @p3, qw/-width 2/);
  $diag->create("line", @p3, @p4, @p1, qw/-width 1/);
	
  my $tiefe = "$divedepth [m]";
  my $time = "$divetime [min]";
	
  # --- print tauchtiefe
  $diag->create('text', @p2, -text => $tiefe, -anchor => 'nw');
  # $diag->create('text', @p3, -text => $time, -anchor => 'nw');

  $tiefe = ($maxy - 0.5) / $divedepth;

  my $x = 0;

  ## ----- Linien --------
  while ( $dekostopps[$x] > 0 ) {
  $x++;
  $ypos = ($dekostopps[$x] * $tiefe) + 0.5;

  $diag->create("line", "0.5c", $ypos."c", $maxx."c", $ypos."c"  , qw/-width 1 -fill yellow/);
  $diag->create('text',  $maxx."c", $ypos."c", -text => $dekostopps[$x]."[m]", -anchor => 'nw');
   
  if ($dekostopps[$x] == 3) {
    goto weiter;
  }
	 
   $x++;
 }

  weiter:

  # ---- Kurve -----
  open(D, "$profilefile") || die "Fehler beim lesen der Datei $profilefile\n";

  my $pdepth = 0;
  my $ptime = 0;
  my $xtime = 0;
  my $xpos = 0.5;
  my $oldx = 0.5;
  my $oldy = 0.5;
  my $adp = 0;
  my $nullz = 0;
	
  # ---- configfile ist gleich ein perlprogramm ! --------
  @myprofile = ();
	
  $x = $o2 = $n2 = $he = 0;
	
  # --- Grafik beginnt bei 0m ------
  $myprofile[$x] = 0.0;	$x++;
  $myprofile[$x] = 0.0;	$x++;
  $myprofile[$x] = 0.0;	$x++;
  while ( <D> ) {
    if (! /#/ ) {
      # nullzeit wird hier garnicht ausgewertet, gaswerte auch nicht
      ($pdepth, $ptime, $adp, $nullz, $o2, $n2, $he) = split(/ +/, $_);
	
 	$myprofile[$x] = $pdepth;
	$x++;
	$myprofile[$x] = $ptime;
	$x++;
	$myprofile[$x] = $adp;
	$x++;

	$xtime += $ptime;
     } else
     {
	  # falls schon Deko, nehme letzte Tiefe zurck,
	# sonst ist ein Haken im Tauchprofil.
	if ( $x )
	{
	  $x -= 3;
	}
     }
  };

  close(D);

  #--- Fehlerbehandlung ---
  if ($xtime <= 0.001)
  {
    my $error = $top->messageBox('-icon' => 'error', -type => 'OK',
		-title => 'ErrorBox', -message => "Dekostop mit <= 0 Minuten, Abbruch der Diagrammausgabe.");
    goto ende;		
  }
  
  # --- Grafik endet bei 0m ------
  $myprofile[$x] = 0.0;	$x++;
  $myprofile[$x] = 0.0;	$x++;
  $myprofile[$x] = 0.0;	$x++;
  
  $printtime = sprintf("%3.0f [min]", $xtime);
  $diag->create('text', @p3, -text => $printtime, -anchor => 'nw');
	
  @linestr = "line";

  $time = ($maxx - 0.5) / $xtime;

  my $y = 0;
  my $timec = 0;
  my $xold = 0;
  my $yold = 0;
  my $nullsave = 0;
  my $deko = 0;

	
  while ( $y < $x ) {
    $ypos = $myprofile[$y] * $tiefe + 0.5;
    $y++;
	      
    $timec += $myprofile[$y];
    $xpos = $timec * $time + 0.5;
    $y++;
	      
    $adp = $myprofile[$y];
    $y++;
	      
    if ( (abs( $xpos - $xold ) > 0.1) || ( abs( $ypos - $yold ) > 0.1 ))
    {
       push(@linestr, $xpos."c");
       push(@linestr, $ypos."c");

       $xold = $xpos;
       $yold = $ypos;
	      
       if ($adp != 0)
       {
 	 if ($deko == 0)
	 {
	   $deko = 1;
	   push(@linestr, qw/-width 2 -fill blue/);
	   $diag->create(@linestr);
	   @linestr = "line";
	   push(@linestr, $xpos."c");
	   push(@linestr, $ypos."c");
         }
	 else
	 {
	   # deko == 1
	   push(@linestr, $xpos."c");
	   push(@linestr, $ypos."c");
	 }
	 # fr die Uhrzeit
	 if ($nullsave == 0)
	 {
	   $nullsave = $timec;
	 }
       }
       if ($deko == 1 && $adp == 0)
       {
         $deko = 0;
	 push(@linestr, qw/-width 2 -fill red/);
	 $diag->create(@linestr);
	 @linestr = "line";
	 push(@linestr, $xpos."c");
	 push(@linestr, $ypos."c");
	}
      }
   }
	
	# Nullzeit Ende 
  my $xnullende = $time * $nullsave + 0.5;
  if ($xnullende <= 1.5)
  {
    $xnullende = 1.5;
  }
  
  if ($nullsave  > 0 )
  { 
    @p0 = ($xnullende."c", $maxy."c");
    $printtime = sprintf("%3.0f [min]", $nullsave);
    $diag->create('text', @p0, -text => $printtime, -anchor => 'nw'); 
  }
  
  # Kurve blau oder rot
  if ( $nullsave > 0 )
  {
    # Taucher Bild ?
    $bild_label = $diag->createImage($xbild, $ybild, -image => 'bildchen');
    push(@linestr, qw/-width 2 -fill red/);
  }
  else
  {
    $bild_label = $diag->createImage($xbild, $ybild, -image => 'fischli');
    push(@linestr, qw/-width 2 -fill blue/);
  }

  $diag->create(@linestr);

 ende:
}

# Drucken und auf LPR ausgeben
sub printdiag {
	printdatei(@_);
	system "$LPRCMD $LPRFILENAME"|| die "Konnte $LPRCMD nicht starten!\n";
}

# nur in Datei drucken
sub printdatei {
	my($w) = @_;
	$a = $w->postscript;
	open($LPR, ">".$LPRFILENAME) || die "Kann Datei $LPRFILENAME nicht anlegen!\n";
	print $LPR $a;
	close $LPR;
}

sub printasc {
	system "$LPRCMD $dekoausg "|| die "Konnte $LPRCMD nicht starten!\n";
}

# ------------------ Tauchprofil Editor --------------------------------------

# speichert liste mit Zahlenpaaren tiefe zeit
@filetext = ();
$tiefe = 10;
$tiefe_old = 0;
$min_old = 0;

sub savetext {
	my($w) = $_[0];
	my $text = $w->get('0.0', 'end');
	@filetext = split(/\n/, $text);
	open(RPF, ">".$realprofile) || die "Kann Datei $realprofile nicht anlegen!\n";
	print RPF $text;
	close(RPF);
}

sub readtext {
	@filetext = ();
	open(RPF, "$realprofile") || return;
 	while ( <RPF> )	
	{
	  !/^\n/ && push (@filetext, $_);
	}
	close(RPF);
}

# --- fge zeiten hinzu ----------
sub bleibeprof {
 # scan variable + speichern kann nicht schaden
 savetext($_[0]);
 
 my $len = @filetext;
 
 $lastentry = $filetext[$len -1];
 
 ($tiefe_old, $min_old) = split(/ /, $lastentry);
 
 my $minstep = 1.0;
 my $line = "";
 my $minende = $min_old + $tiefe;
 
 while ( $min_old < $minende )
 {
   $min_old += $minstep;
	 $line = sprintf("%2.2f %2.2f\n", $tiefe_old, $min_old);
	 $_[0]->insert('end', $line);
 }
 
}

# --- fge tiefen hinzu ----------
sub addprof {
 # scan variable + speichern kann nicht schaden
 savetext($_[0]);
 
 my $len = @filetext;
 
 $lastentry = $filetext[$len -1];
 
 ($tiefe_old, $min_old) = split(/ /, $lastentry);
 

 my $tiefestep = $_[1];
 my $minstep = 0.1;
 my $speed   = 10;

 my $min = $tiefe / $speed;
 my $minende = $min_old + $min;
 
 my $line = "";
 
 while ( $min_old < $minende )
 {
   $min_old += $minstep;
   $tiefe_old += $tiefestep;
   $line = sprintf("%2.2f %2.2f\n", $tiefe_old, $min_old);
   $_[0]->insert('end', $line);
 }
 
}

#-------------------- Tauchprofile eingeben ---------------------------------

sub editintern {
  
# erst text lesen
  readtext();
	
  my $toplevel = MainWindow->new;
  $toplevel->setPalette($palette);
	  
  $toplevel->title("Profile eingeben");
	
  my(@pu) = qw/-side top -expand yes -pady 1m/;
  my $toplabel = $toplevel->Label(-text => 'Tauchprofil eingeben:')->pack(@pu);
	
  my $topl = $toplevel->Frame(-relief => 'raised', -borderwidth => 1)->pack(@pu);

  my $textwdg = $topl->Scrolled(qw/Text -width 50 -setgrid true -scrollbars e/);
  $textwdg->pack(qw/-side left -fill y/);

  for (my $x=0; $zeile = $filetext[$x]; $x++)
  {
     $textwdg->insert('end', $zeile);
  }
	
  my @pl = qw/-side left -padx 2m -pady 2m/;

  my $fe = $toplevel->Frame->pack(@pu);
  my $tiefe = $fe->Entry(-relief => 'sunken', -width => 4, -textvariable => \$tiefe)->pack(@pl);
										
  my $l1 = $fe->Label(-text => '[m|min]')->pack(@pl);
	
  my $aufstg = $fe->Button( -text => 'Abstieg',
			-command => [\&addprof, $textwdg, 1])
			->pack(@pl);
  my $abstg = $fe->Button( -text => 'Aufstieg',
			-command => [\&addprof, $textwdg, -1])
			->pack(@pl);

  my $verbl = $fe->Button( -text => 'Verbleiben',
			-command => [\&bleibeprof, $textwdg])
			->pack(@pl);
					
  my $f7 = $toplevel->Frame->pack(@pu);

  my $save = $f7->Button( -text => 'Speichern',
			-command => [\&savetext, $textwdg])
			->pack(@pl);
	
  my $ok = $f7->Button( -text => 'Zu',
			-command => sub { @filetext = (); destroy $toplevel; })
			->pack(@pl);
}


#------------------------- Tabellenberechnung ---------------------------

$tab_von = 15;
$tab_bis = 67;
$tab_step = 5;
$tab_vonmin = 15;
$tab_bismin = 90;
$tab_stepmin = 10;
$tab_maxlen = 120;

sub calc_tg {
  $divedepth = $_[0];
  $divetime = $_[1];
  # my $tiefe = $_[0];
  # my $zeit = $_[1];

  # globaler paramstr ist auch ok
  #@paramstr = "--depth $tiefe --time $zeit";
  @paramstr = "--depth $divedepth --time $divetime";

  make_parameterstring();

  # zum Datenaustausch wird ein File angelegt mit dem Tauchprofiel
  # push(@paramstr, "--profile-out --save-rest");
  push(@paramstr, "--profile-out ");

  $systemstr = "$dekoprog @paramstr > $dekoausg";
  print "system: $systemstr \n";

  # Teste ob das dekoprog schon compiliert ist.
  if ( -e $dekoprog )
  {
    # ----- Programmstart !!! ----------------------
    system $systemstr || die "konnte $systemstr nicht starten!\n";
  }
  else
  {
    print "compiliere $dekoprog mit make -C $instpath!\n";
    system "make -C $instpath" || die "konnte $dekoprog nicht compilieren!\n";
    # ----- Programmstart !!! ----------------------
    system $systemstr || die "konnte $systemstr nicht starten!\n";
  }
  
  open (IN, $dekoausg) || die "Kann Datei $dekoausg nicht ffnen!\n";
	
  my $x = 0;
  my $sum = 0;
  my $zeit = 0;

  @dekostopps = ();

  while ( <IN> ) 
  {
	# 1. Deko stopp
	if ( /min auf/ )
	{
		($zeit, $rest) = split(/min auf/, $_);
		($meter, $rest) = split(/ m/, $rest);
		$sum += $zeit;
		$dekostopps[$x] = sprintf ("%2d->%2d|", $zeit, $meter);
		$x++;
	}

  }
  # damit beim zeichnen die while schleife abgebrochen wird
  close IN;
  return $sum;
}

sub calctable {

  my %tabellen = ();
	
  my $deko = $z = 0;
  my $tabstr = "";
  #  ++++++ Tabelle berechnen +++++++++++++++++++++++++
  #  
  for (my $x=$tab_von; $x <= $tab_bis; $x += $tab_step)	
  {
    for (my $y=$tab_vonmin; $y <= $tab_bismin; $y += $tab_stepmin) 
    {
      # --- Tauchgangberechnung ---
	$deko = calc_tg($x, $y);
	if ( ($deko + $y) <= $tab_maxlen ) 
	{
	    $z++;
	    $tabstr = sprintf("%3d|%5d|", $x, $y);
	    $tabellen{$z} = "$tabstr @dekostopps";
	}
	else 
	{
	  $y = $tab_bismin +1; # Abbruch
	}
     }
    $z++; 
    $tabellen{$z} =  "-------------------------------------------------|";
  }

  # ----- Grafik ausgeben -----
  neue_grafik();
  
  # ++++++++ Textfelder bauen ++++++++++++++++++++++++++
  $text->delete(qw/1.0 end/);
  $text->insert('0.0', "|------- Tauchtabellen Berechnet mit X-Deko ------|\n");
      my $tablinestr = "|-------------------------------------------------|\n";
  $text->insert('end', $tablinestr);
  $text->insert('end', "|  m|  min| min->m| min->m| min->m| min->m| min->m|\n");
             $tabstr = "|---|-----|-------|-------|-------|-------|-------|\n";
  $text->insert('end', $tabstr);

  for (my $x = 1; $x <= $z; $x ++) 
  {
    # ### Tabelle ist fertig, nur noch ins canvas zeichnen !
    #$tabstr = sprintf("%4d |", $x);
    $text->insert('end', "|$tabellen{$x}\n");
    # $text->insert('end', $tabstr);
  }
  # tiefe und Zeit lschen
  @paramstr = "";
  make_parameterstring();
  $text->insert('end', "|Parameter: @paramstr\n");
  $text->insert('end', $tablinestr);

  save_tabelle();

  #  +++++++ Statuszeile ++++++++++++++
  $statustext = "Berechnete Tabellen: $z von: $tab_von [m] bis: $tab_bis [m] parm: @paramstr";
}

sub save_tabelle {
  # #### speichern zum drucken
  my $strtext = $text->get('0.0', 'end');
  open(D, ">$tableausg") || die "Fehler beim schreiben der Datei $tableausg\n";
  print D $strtext;
  close D;
}

sub print_tabelle {
  # damit die Editierte Tabelle auch gedruckt wird
  save_tabelle();
  system "$LPRCMD $tableausg "|| die "Konnte $LPRCMD nicht starten!\n";
}

sub tabelle {
  
  my $toplevel = MainWindow->new;
  $toplevel->setPalette($palette);
  $toplevel->title("Tabellenparameter");
	
  my(@pu) = qw/-side top -expand yes /;
  my $toplabel = $toplevel->Label(-text => 'Tabelle eingeben:')->pack(@pu);
	
  my $topl = $toplevel->Frame(-relief => 'raised', -borderwidth => 1)->pack(@pu);


  my @pl = qw/-side left -padx 1m -pady 1m/;
	
  my $fe = $toplevel->Frame->pack(@pu);
  my $l1 = $fe->Label(-text => 'Von [m]:')->pack(@pl);
  my $tvon = $fe->Entry(-relief => 'sunken', -width => 4, -textvariable => \$tab_von)->pack(@pl);

  my $fe1 = $toplevel->Frame->pack(@pu);
  my $l11 = $fe1->Label(-text => 'Bis [m]:')->pack(@pl);
  my $tbis = $fe1->Entry(-relief => 'sunken', -width => 4, -textvariable => \$tab_bis)->pack(@pl);

  my $fe2 = $toplevel->Frame->pack(@pu);
  my $l12 = $fe2->Label(-text => 'Schrittweite [m]:')->pack(@pl);
  my $tstep = $fe2->Entry(-relief => 'sunken', -width => 4, -textvariable => \$tab_step)->pack(@pl);

  my $fe3 = $toplevel->Frame->pack(@pu);
  my $l13 = $fe3->Label(-text => 'Von [min]:')->pack(@pl);
  my $mvon = $fe3->Entry(-relief => 'sunken', -width => 4, -textvariable => \$tab_vonmin)->pack(@pl);

  my $fe4 = $toplevel->Frame->pack(@pu);
  my $l14 = $fe4->Label(-text => 'Bis [min]:')->pack(@pl);
  my $mbis = $fe4->Entry(-relief => 'sunken', -width => 4, -textvariable => \$tab_bismin)->pack(@pl);

  my $fe5 = $toplevel->Frame->pack(@pu);
  my $l15 = $fe5->Label(-text => 'Schrittweite [min]:')->pack(@pl);
  my $mstep = $fe5->Entry(-relief => 'sunken', -width => 4, -textvariable => \$tab_stepmin)->pack(@pl);
	
  my $fe6 = $toplevel->Frame->pack(@pu);
  my $l16 = $fe6->Label(-text => 'Tauchgang Max. Lnge [min]:')->pack(@pl);
  my $mlen = $fe6->Entry(-relief => 'sunken', -width => 4, -textvariable => \$tab_maxlen)->pack(@pl);

  my $f7 = $toplevel->Frame->pack(@pu);
  my $calc = $f7->Button( -text => 'Berechnen',
			-command => [\&calctable ])
			->pack(@pl);
  $calc->focus();	
  my $ok = $f7->Button( -text => 'Zu',
			-command => sub { @filetext = (); destroy $toplevel; })
			->pack(@pl);
}


# --- Tk ---
MainLoop;
0;


