#!/usr/bin/perl -w

############################################################
# Programm zum Erstellen von UDCF Dateien.
# Autor: Helmut Fahrion
# E-Mail: hefa@gmx.net
# Lizenz: GPL
# Datum: 12.2003
# #############################################################

use XML::Writer;
use IO;


# --------- Default Gas -----------
$switch_gas = "Luft";

# --------- fr addinfo -----------
$persinfo   = 'hefa@gmx.net';
$totaldives = "180";
$serialid   = "241065";
$prg_data   = "X-Deko Simulation";

# ---- Pfad wo die Programm und Dokudateien sind ----------------------------
$instpath	= "$ENV{HOME}/deko";

# --- Falls es keine lokale Installation gibt -------------------------------
if (! -e $instpath)
  {
    $instpath	= "/opt/deko";
  }


if ($ARGV[0])
{
  # ------- Ausgabedatei -----------
  $outfile    = $ARGV[0];
}
else
{
  # --- Default, im aufrufenden Verzeichnis ----
  $outfile = "deko-erg.udcf";
}

# --------- Inputdatei -------------
$eingabefile = "deko-profil.txt";
$ergfile = "deko-erg.txt";

# --------- Versionsnummer -------------
$versionsfile   = "$instpath/deko-version.h";
open(D, $versionsfile) || die "Fehler beim lesen der Datei $versionsfile\n";
while ( <D> ) {
   if ( /#define version/ )
   {
     /"(.*)"/;
     $version = $1;
   }
}
close(D);

   
# ------------ Datum -------------------

($dummy,$min,$hour,$mday,$mon,$year,$dummy,$dummy,$dummy) = localtime(time);
$mon += 1;
$year += 1900;
$dummy = 0;

# ------------- Altitude, Meereshhe ------------
$altitude = 0;
# ------------- Oberflchenpause in Sekunden ------------
$ofp_secs = 0;

# ------------- Konstantes ppO2 fr Rebreather ----------
$const_ppo2 = 0.0;


# ---------- Verwendete Gase ------------

# ------------- Ergebnisdatei einlesen ------------------
# --- Daten fr das udcf File ---
open (IN, $ergfile) || die "Kann Datei $ergfile nicht ffnen!\n";
while ( <IN> ) {
   # print $_;

   if ( /Sauerstoff:/ )
   {
   	$gases{"O2"}   = "100 0 0 4 200 50";
   }
   
   if ( /Luft:/ )
   {
   	$gases{"Luft"}   = "21 79 0 10 200 50";
   }
   
   if ( /Nitrox:/ )
   {
   	($gasname, $prozento2, $rest) = split(" +", $_);
	$n2 = 100 - $prozento2;
   	$gases{"Nitrox"}   = "$prozento2 $n2 0 10 200 50";
   }
   
   if ( /Trimix:/ )
   {
   	($gasname, $prozenthe, $rest) = split(" +", $_);
	$o2 = 0.21 * (100.0 - $prozenthe);
	$n2 = 0.79 * (100.0 - $prozenthe);
   	$gases{"Trimix"}   = "$o2 $n2 $prozenthe 10 200 50";
   }

   # ------------- Oberflchenpause in Sekunden ------------
   if ( /Oberfl.Pause:/ )
   {
   	($rest, $ofp_h, $rest) = split(" +", $_);
	$ofp_secs = $ofp_h * 60.0 * 60.0;
   }
   
   # ------------- Bergseetauchen ?  ------------
   if ( /Hoehe:/ )
   {
   	($rest, $altitude, $rest) = split(" +", $_);
   }
   
   # ------------- ppO2 ------------
   if ( /ppO2:/ )
   {
      ($rest, $const_ppo2, $rest) = split(" +", $_);  
   }

}
close IN;


# ------------- Density Ssswasser festgelegt ------------
$density = 1000.0;


# installiere den writer
$output = new IO::File(">$outfile");
$writer = new XML::Writer(OUTPUT => $output);


# ------ Profile Section ------------------------------------------
$writer->startTag('profile', 'udcf' => '1');
$writer->characters("\n");

$writer->comment("Datei fuer den Datenaustausch.");
$writer->characters("\n");

# 
# ------ Programm Beginn  ----------------------------------------
units_section();
device_section();
additional_section();
rep_groups();

$writer->endTag('profile');
$writer->end();
# ------ Programm Ende  ------------------------------------------

# ------ Unterfunktion  ------------------------------------------
sub tab_dataElement
{
  $writer->characters("\t");
  $writer->dataElement($_[0], $_[1]);
  $writer->characters("\n");
}


# ------ Units Section ------------------------------------------
sub units_section
{
  $writer->comment("Masssystem fuer Europa");
  $writer->characters("\n");
  $writer->dataElement('units', "Metric");
  $writer->characters("\n");
}

# ------ Device Section ------------------------------------------
sub device_section
{
  $writer->startTag('device');
  $writer->characters("\n");
  vendor_section();
  model_section();
  version_section();
  $writer->endTag('device');
  $writer->characters("\n");
}
# ------ Vendor Section ------------------------------------------
sub vendor_section
{
  tab_dataElement('vendor', "Helmut Fahrion, Privat.");
}
# ------ Model Section ------------------------------------------
sub model_section
{
  tab_dataElement('model', "X-Deko, Linux Software.");
}
# ------ Version Section ------------------------------------------
sub version_section
{
  tab_dataElement('version', "$version");
}

# ------ Version Section ------------------------------------------
sub additional_section
{
  $writer->startTag('addinfo');
  $writer->characters("\n");
  personal_desc();
  totalDives_desc();
  IDString_desc();
  $writer->endTag('addinfo');
  $writer->characters("\n");
}
sub personal_desc
{
  tab_dataElement('persinfo', $persinfo);
}
sub totalDives_desc
{
  tab_dataElement('totaldives', $totaldives);
}
sub IDString_desc
{
  tab_dataElement('serialid', $serialid);
}


# ------ Rep Groups ------------------------------------------
# sich widerholende Gruppen / Tauchgnge
sub rep_groups
{
  $writer->startTag('repgroup');
  $writer->characters("\n");
  divelist();

  $writer->endTag('repgroup');
  $writer->characters("\n");
}

# ------ Tauchgangslist ---------------------------------------
sub divelist
{
  $writer->startTag('dive');
  $writer->characters("\n");

  place_desc();
  date();
  dive_time();
  surface_interval();
  
  temperature();
  density();
  altitude();
  gases_section();

  prg_data();

  samples();

  $writer->endTag('dive');
  $writer->characters("\n");
}

sub prg_data
{
  $writer->dataElement('prg', $prg_data);
  $writer->characters("\n");
}

sub place_desc
{
  $writer->dataElement('place', "Simulator");
  $writer->characters("\n");
}

sub date
{
  $writer->comment("Datum der Simulation");
  $writer->characters("\n");
  $writer->startTag('date');
  $writer->characters("\n");

  $writer->characters("\t");
  $writer->dataElement('year', $year);
  $writer->dataElement('month', $mon);
  $writer->dataElement('day', $mday);
  $writer->characters("\n");

  $writer->endTag('date');
  $writer->characters("\n");
}

sub dive_time
{
  $writer->startTag('time');
  $writer->characters("\n");

  $writer->characters("\t");
  $writer->dataElement('hour', $hour);
  $writer->dataElement('minute', $min);
  $writer->characters("\n");

  $writer->endTag('time');
  $writer->characters("\n");
}

sub surface_interval
{
  $writer->comment("Oberflaechenpause in Sekunden");
  $writer->characters("\n");
  $writer->dataElement('surfaceinterval', $ofp_secs);
  $writer->characters("\n");
}

sub temperature
{
  $writer->comment("Suesswassertemperature");
  $writer->characters("\n");
  $writer->dataElement('temperature', "4.0");
  $writer->characters("\n");
}

sub density
{
  $writer->comment("Suesswasser");
  $writer->characters("\n");
  $writer->dataElement('density', $density);
  $writer->characters("\n");
}

sub altitude
{
  $writer->comment("Hoehenmeter");
  $writer->characters("\n");
  $writer->dataElement('altitude', $altitude);
  $writer->characters("\n");
}

sub gases_section
{
  $writer->startTag('gases');
  $writer->characters("\n");

  mix_list();
  
  $writer->endTag('gases');
  $writer->characters("\n");
}

# ------ Liste der Gasbeschreibungen ------
sub mix_list
{
  
  foreach (keys %gases) {
    mix_desc( $_ );
  }
  
}

sub mix_desc
{
  $writer->startTag('mix');
  $writer->characters("\n");

  # --- Gemischname ----
  $writer->characters("\t");
  $writer->dataElement('mixname', $_[0]);
  $writer->characters("\n");
  
  my ($md_o2, $md_n2, $md_he, $md_vol, $md_press, $md_res) = 
  		split(" ", $gases { $_[0] });
  
  tank_info($md_vol, $md_press, $md_res);

  $md_o2 /= 100.0;
  $md_n2 /= 100.0;
  $md_he /= 100.0;

  # selbst hinzugefgt
  if ( $const_ppo2 > 0 )
  {
    tab_dataElement('ppo2', $const_ppo2);
  }  
  tab_dataElement('o2', $md_o2);
  tab_dataElement('n2', $md_n2);
  tab_dataElement('he', $md_he);
  $writer->characters("\n");
  
  $writer->endTag('mix');
  $writer->characters("\n");
}


sub tank_info
{
  $writer->characters("\t");
  $writer->startTag('tank');
  $writer->characters("\n");

  $writer->characters("\t");
  tank_volume_info($_[0]);
  
  $writer->characters("\t");
  tank_volume_pressin($_[1]);

  $writer->characters("\t");
  tank_volume_pressres($_[2]);

  $writer->characters("\t");
  $writer->endTag('tank');
  $writer->characters("\n");
}

sub tank_volume_info
{
  tab_dataElement('tankvolume', $_[0]);
}
sub tank_volume_pressin
{
  tab_dataElement('pstart', $_[0]);
}
sub tank_volume_pressres
{
  tab_dataElement('pend', $_[0]);
}

# ----- das eigentliche profile ------
sub samples
{
  # es folgen zeit und tiefenangaben
  $writer->emptyTag("timedepthmode");
  $writer->characters("\n");
  

  $writer->startTag('samples');
  $writer->characters("\n");

  # Gas Switch eintragen
  # switch_desc( "Luft" );
  
  sample_list_timedepth();

  $writer->endTag('samples');
  $writer->characters("\n");
}


# ---------- Gaswechsel -----------
sub switch_desc
{
  $switch_gas = $_[0];
  tab_dataElement('switch', $_[0]);
}

# ----------  Finde Gasswitch ----------
sub check_gas
{
  $o2 = $_[0];
  $n2 = $_[1];
  $he = $_[2];
  
  if ($he > 0)
  {
    if ( $switch_gas ne "Trimix"  )
    {
      switch_desc( "Trimix" );
      return;
    }
  }

  if ($o2 == 100)
  {
    if ( $switch_gas ne "O2"  )
    {
      switch_desc( "O2" );
    }  
  }
  if ($o2 > 21 && $o2 < 100 && $he <= 0)
  {
    if ( $switch_gas ne "Nitrox"  )
    {
     switch_desc( "Nitrox" );
    }
  }
  if ($n2 == 79 && $o2 == 21)
  {
    if ( $switch_gas ne "Luft"  )
    {
      switch_desc( "Luft" );
    }  
  }
  
}


sub sample_list_timedepth
{
 # sampledesc_timedepth sample_list_timedepth
 #                         | sampledesc_timedepth

 # beginnt immer mit 0.0 und 0.0 
 sampledesc_timedepth("0.0", "0.0"); 

 # ------------- Profildatei einlesen ------------------
 # --- Daten fr das udcf File ---
 $dive_time = 0.0;
 
 open (IN, $eingabefile) || die "Kann Datei $eingabefile nicht ffnen!\n";
 while ( <IN> ) {
    if (! /#/ ) {
    
      ($depth, $time, $rest, $rest, $o2, $n2, $he) = split(" +", $_);

      $dive_time += $time;
      check_gas($o2, $n2, $he);
      sampledesc_timedepth($dive_time, $depth); 
    }
 }
 close IN;
}

sub sampledesc_timedepth
{
#  sampledesc_timedepth ->   point
#    | switch_desc
#    | temperature
#    | markers

#  point -> <T> Real </T>
#   <DEPTH> Real </DEPTH> 
  $pt = sprintf("%3.1f", $_[0]); 
  $pd = sprintf("%3.1f", $_[1]);
  $writer->characters("\t");
  $writer->dataElement('t', $pt);
  $writer->dataElement('d', $pd);
  $writer->characters("\n");
}


$output->close();

# Ende !!! :-)


